@@ -35,7 +35,7 @@
 #include <math.h>
 #include <string.h>
 
-#include "casu_utils.h"
+#include "catalogue/casu_utils.h"
 #include "casu_filt.h"
 
 /* Structure for holding useful information for the stats routine */
