@@ -88,7 +88,7 @@ static char *form_request(float ra, float dec, float dra, float ddec,
                           float equinox, const char *catid);
 static char *url_encode(char *instring);
 static int connect_site(const char *site, int *sock);
-static int send_request(int sock, char *req_string);
+static int send_request(int sock, char *req_string, int isbin);
 static int get_response(int soc, cpl_table **outtab);
 
 /**@{*/
@@ -166,6 +166,7 @@ extern int casu_getstds(cpl_propertylist *plist, int cache, char *path,
     int n,i,sock;
     cpl_propertylist *p;
     char *req_string,catname2[64],*cdscatname,*cdscatid,*path2;
+    int isbin = (cdssearch > 6 ? 1 : 0);
 
     /* Inherited status */
 
@@ -187,6 +188,8 @@ extern int casu_getstds(cpl_propertylist *plist, int cache, char *path,
     ramid = 0.5*(ramax+ramin);
     decmid = 0.5*(decmax+decmin);
 
+    cpl_msg_info(fctid, "Searching catalogue %d, RA/Dec range [%f:%f, %f:%f]", cdssearch, ramin, ramax, decmin, decmax);
+
     /* If using the cache, then check if for a previously used table */
 
     cdscatid = NULL;
@@ -211,6 +214,8 @@ extern int casu_getstds(cpl_propertylist *plist, int cache, char *path,
         /* Read the standards from local catalogues */
 
         if (cdssearch == 0) {
+            cpl_msg_info(fctid, "Reading local catalogue");
+
             path2 = cpl_strdup(path);
             if (! strcmp(catname2,"casu_2mass")) {
                 *stds = casu_2mass_extract(dirname(path2),ramin,ramax,decmin,
@@ -233,6 +238,8 @@ extern int casu_getstds(cpl_propertylist *plist, int cache, char *path,
         /* Read the standards from CDS */
 
         } else {
+            cpl_msg_info(fctid, "Requesting catalogue data from CDS");
+
             req_string = form_request(ramid,decmid,dra,ddec,2000.0,
                                       cdscatid);
             freespace(cdscatid);
@@ -246,7 +253,7 @@ extern int casu_getstds(cpl_propertylist *plist, int cache, char *path,
             
             /* Send the request to the site */
 
-            if (send_request(sock,req_string) != CASU_OK) {
+            if (send_request(sock,req_string,isbin) != CASU_OK) {
                 cpl_msg_warning(fctid,"Couldn't send request to site CDS");
                 FATAL_ERROR
             }
@@ -277,11 +284,13 @@ extern int casu_getstds(cpl_propertylist *plist, int cache, char *path,
         freespace(cdscatid);
     }
 
+    n = (int)cpl_table_get_nrow(*stds);
+    cpl_msg_info(fctid, "Found %d stars in catalogue", n);
+
     /* If there are no rows in the table, this may be a cause for concern.
        So add the columns into the table and then set a warning return 
        status */
 
-    n = (int)cpl_table_get_nrow(*stds);
     if (n == 0) {
         cpl_table_new_column(*stds,"xpredict",CPL_TYPE_FLOAT);
         cpl_table_new_column(*stds,"ypredict",CPL_TYPE_FLOAT);
@@ -377,13 +386,14 @@ extern int casu_getstds(cpl_propertylist *plist, int cache, char *path,
  */
 /*---------------------------------------------------------------------------*/
 
+static const int maxcds = 7;
+static const char *cdscatnames[8] =
+    {"","2mass","usnob","ppmxl","landolt","wise", "apass", "gaiaedr3"};
+static const char *cdscats[8] = 
+    {"","II/246","I/284","I/317","II/183A","II/311", "II/336", "I/350"};
+
 extern int casu_getstds_cdslist(int cdschoice, char **cdscatname,
                                 char **cdscatid, int *status) {
-    const char *cdscats[7] = {"","II/246","I/284","I/317","II/183A","II/311",
-                              "II/336"};
-    const char *cdscatnames[7] = {"","2mass","usnob","ppmxl","landolt","wise",
-                                  "apass"};
-    const int maxcds = 6;
     const char *fctid = "casu_getstds_cdslist";
 
     if (*status != CASU_OK) 
@@ -402,6 +412,45 @@ extern int casu_getstds_cdslist(int cdschoice, char **cdscatname,
     *cdscatid = cpl_strdup(cdscats[cdschoice]);
     GOOD_STATUS
 }
+
+/*---------------------------------------------------------------------------*/
+/**
+    \par Name:
+        casu_get_cdschoice
+    \par Purpose:
+        Get the internal integer that matches the designation of CDS
+        catalogues served through the casu_getstds interface
+    \par Description:
+        Looks up the appropriate integer number representing the choice
+        of CDS catalogue.
+    \par Language:
+        C
+    \param cdscatname
+        The CDS designation for the catalogue
+    \retval 0
+        if the string doesn't match a catalogue choice
+    \retval 1 .. maxcds 
+        the matching catalogue choice
+    \par QC headers:
+        None
+    \par DRS headers:
+        None
+    \author
+        Jon Nielsen
+ */
+/*---------------------------------------------------------------------------*/
+
+extern int casu_get_cdschoice(const char* cdscatname)
+{
+    const char *fctid = "casu_get_cdschoice";
+
+    for (int i=1; i<=maxcds; i++) {
+        if (!strcmp(cdscatname, cdscatnames[i])) {
+            return i;
+        }
+    }
+    return 0;
+}
         
 
 /*---------------------------------------------------------------------------*/
@@ -1181,6 +1230,9 @@ static cpl_table *check_cache(char *catname, float ra1_im, float ra2_im,
 
     out_cat = cpl_table_load(cat_cache,1,1);
     cpl_free(cat_cache);
+
+    cpl_msg_info(cpl_func, "Using cached catalogue data");
+
     return(out_cat);
 }
     
@@ -1378,12 +1430,16 @@ static int get_response(int sock, cpl_table **outtab) {
     \par Description:
         A given request string is sent through to the CDS through
         a previously opened socket.
+        Requests either an ascii or binary table depending on the
+        isbin parameter.
     \par Language:
         C
     \param sock
         The input socket number
     \param req_string
         The request string
+    \param isbin
+        requests a binary table if this is true
     \retval CASU_OK
         if all went well
     \retval CASU_FATAL
@@ -1393,13 +1449,17 @@ static int get_response(int sock, cpl_table **outtab) {
  */
 /*---------------------------------------------------------------------------*/
         
-static int send_request(int sock, char *req_string) {
+static int send_request(int sock, char *req_string, int isbin) {
     char buf[BUFMAX];
     const char *fctid = "send_request";
 
     /* Form the full request contents and send it. */
 
-    sprintf(buf,"GET /viz-bin/asu-fits?%s HTTP/1.0\r\n\r\n",req_string);
+    if (isbin) {
+        sprintf(buf,"GET /viz-bin/asu-binfits?%s HTTP/1.0\r\n\r\n",req_string);
+    } else {
+        sprintf(buf,"GET /viz-bin/asu-fits?%s HTTP/1.0\r\n\r\n",req_string);
+    }
     if (send(sock,buf,strlen(buf),0) < 0) {
         cpl_msg_warning(fctid,"Attempt to send message failed, error: %d\n",
                         errno);
