@@ -37,9 +37,9 @@
 #include <string.h>
 
 #include "casu_mods.h"
-#include "casu_utils.h"
+#include "catalogue/casu_utils.h"
 #include "casu_stats.h"
-#include "casu_fits.h"
+#include "catalogue/casu_fits.h"
 
 typedef struct {
     char *filter_name;
@@ -211,7 +211,7 @@ extern int casu_photcal_extinct(casu_fits **images, casu_tfits **mstds,
     int ngood,oops,nlim,isrubbish,*dodgy,crud;
     const char *fctid = "casu_photcal_extinct";
     casu_fits *im;
-    cpl_propertylist *ehu_im,*ehu_cat,*phu_im;
+    cpl_propertylist *ehu_im,*ehu_cat,*phu_im,*phu_cat;
     cpl_table *stds;
     cpl_image *schl_n,*schl_s;
     photstrct p;
@@ -307,8 +307,8 @@ extern int casu_photcal_extinct(casu_fits **images, casu_tfits **mstds,
                             "Too few standards available in %" CPL_SIZE_FORMAT,
                             (cpl_size)(i+1));
             cpl_error_reset();
-            write_hdr_1(ehu_im,ncat,0.0,1.0,0.0,0.0,1.0,0.0,0.0,0.0,0.0,1);
-            write_hdr_1(ehu_cat,ncat,0.0,1.0,0.0,0.0,1.0,0.0,0.0,0.0,0.0,0);
+            write_hdr_1(ehu_im,ncat,0.0,1.0,0.0,0.0,1.0,0.0,p.atm_extcoef,0.0,0.0,1);
+            write_hdr_1(ehu_cat,ncat,0.0,1.0,0.0,0.0,1.0,0.0,p.atm_extcoef,0.0,0.0,0);
             dodgy[i] = 1;
             continue;
         }
@@ -344,10 +344,16 @@ extern int casu_photcal_extinct(casu_fits **images, casu_tfits **mstds,
         for (j = 0; j < p.ncolumns_coleq; j++) {
             cpl_table_fill_invalid_float(stds,(p.coleq_columns)[j],blankvalue);
             stdmagptr[j] = cpl_table_get_data_float(stds,(p.coleq_columns)[j]);
+            if (stdmagptr[j] == NULL)
+            {
+                cpl_msg_error(fctid,"Unable to find filter column %s in photometric calibration table", p.coleq_columns[j]);
+                FATAL_ERROR
+            }
+
             if (magerrptr != NULL) {
-		cpl_table_fill_invalid_float(stds,(p.coleq_errcols)[j],blankvalue);
+                cpl_table_fill_invalid_float(stds,(p.coleq_errcols)[j],blankvalue);
                 magerrptr[j] = cpl_table_get_data_float(stds,(p.coleq_errcols)[j]);
-	    }
+            }
         }
 
         /* Get some workspace for the results arrays for this image */
@@ -384,11 +390,11 @@ extern int casu_photcal_extinct(casu_fits **images, casu_tfits **mstds,
                     oops = 1;
                     break;
                 } else if (magerrptr != NULL) {
-		    if (magerrptr[k][j] > magerrcut)  {
-			oops = 1;
-			break;
-		    }
-		}
+                    if (magerrptr[k][j] > magerrcut)  {
+                        oops = 1;
+                        break;
+                    }
+                }
                 refmag += ((p.coleq_coefs)[k]*stdmagptr[k][j]);
             }
             if (oops) 
@@ -445,14 +451,14 @@ extern int casu_photcal_extinct(casu_fits **images, casu_tfits **mstds,
         float lim3 = 0;
         if (! crud) {
             lim3 = casu_calculate_abmag_lim(med3 + 2.5 * log10(exptime) - extinct,
-            		sknoise, rcore, exptime, apcor3, extinct);
+                                            sknoise, rcore, exptime, apcor3, extinct);
             lim5 = med5 - 2.5*log10((5.0*sknoise*2.0*rcore*sqrt(CPL_MATH_PI))/exptime) -
                 apcor5 - extinct;
         } else {
             med3 = p.default_zp;
             med5 = p.default_zp;
             lim3 = casu_calculate_abmag_lim(med3 + 2.5 * log10(exptime) - extinct,
-            		sknoise, rcore, exptime, apcor3, extinct);
+                                            sknoise, rcore, exptime, apcor3, extinct);
             lim5 = med5 - 2.5*log10((5.0*sknoise*2.0*rcore*sqrt(CPL_MATH_PI))/exptime) -
                 apcor5 - extinct;
         }
@@ -503,12 +509,13 @@ extern int casu_photcal_extinct(casu_fits **images, casu_tfits **mstds,
         isrubbish = 0;
     } else {
         med3 = p.default_zp;
-        sig3 = 1.0;
+        sig3 = p.default_zp_err;
         med5 = 0.0;
         sig5 = 1.0;
         medebv = -1.0;
         isrubbish = 1;
     }
+    char const * const fluxcal = isrubbish ? "UNCALIBRATED" : "ABSOLUTE";
 
     /* If there were good images, then calculate the RMS of the detector
        level zeropoints in both apertures */
@@ -518,12 +525,9 @@ extern int casu_photcal_extinct(casu_fits **images, casu_tfits **mstds,
         sig3det = mad*1.48;
         (void)casu_medmad(zps5,NULL,(long)ngood,&med,&mad);
         sig5det = mad*1.48;
-    } else if (ngood == 1) {
+    } else {
         sig3det = sig3;
         sig5det = sig5;
-    } else {
-        sig3det = 0.0;
-        sig5det = 0.0;
     }
     if (nlim > 0) {
         sumlim /= (float)nlim;
@@ -551,14 +555,25 @@ extern int casu_photcal_extinct(casu_fits **images, casu_tfits **mstds,
         im = images[i];
         ehu_im = casu_fits_get_ehu(im);
         ehu_cat = casu_tfits_get_ehu(cats[i]);
+        phu_im = casu_fits_get_phu(im);
+        phu_cat = casu_tfits_get_phu(cats[i]);
+
+        cpl_propertylist_update_string(phu_im,"FLUXCAL",fluxcal);
+        cpl_propertylist_set_comment(phu_im,"FLUXCAL",
+                                     "Certifies the validity of PHOTZP");
+        cpl_propertylist_update_string(phu_cat,"FLUXCAL",fluxcal);
+        cpl_propertylist_set_comment(phu_cat,"FLUXCAL",
+                                     "Certifies the validity of PHOTZP");
 
         /*According to the PHASE 3 standard PHOTZP has to absorb the exposure
          * time and the extinciton*/
-        phu_im = casu_fits_get_phu(im);
         exptime = 1.0; /* Assumption in case of missing keywords */
         extinct = 0.0; /* Assumption in case of missing keywords */
         exptime = (float)cpl_propertylist_get_double(phu_im,expkey);
         extinct = cpl_propertylist_get_float(ehu_im,"ESO DRS EXTINCT");
+        double photzp = 0.0;
+        if(med3 != 0.0)
+            photzp = med3 + 2.5*log10(exptime) - extinct;
 
         cpl_propertylist_update_int(ehu_im,"ESO DRS MAGNZPTALL",nresall);
         cpl_propertylist_set_comment(ehu_im,"ESO DRS MAGNZPTALL",
@@ -593,7 +608,7 @@ extern int casu_photcal_extinct(casu_fits **images, casu_tfits **mstds,
         cpl_propertylist_update_double(ehu_im,"ESO QC SKYBRIGHT",sumskbr);
         cpl_propertylist_set_comment(ehu_im,"ESO QC SKYBRIGHT",
                                      "[mag/arcsec**2] sky brightness");
-        cpl_propertylist_update_double(ehu_im,"PHOTZP",med3 + 2.5*log10(exptime) - extinct);
+        cpl_propertylist_update_double(ehu_im,"PHOTZP",photzp);
         cpl_propertylist_set_comment(ehu_im,"PHOTZP",
                                      "[mag] photometric zeropoint");
         cpl_propertylist_update_double(ehu_im,"PHOTZPER",sig3det);
@@ -652,7 +667,7 @@ extern int casu_photcal_extinct(casu_fits **images, casu_tfits **mstds,
         cpl_propertylist_update_double(ehu_cat,"ESO QC SKYBRIGHT",sumskbr);
         cpl_propertylist_set_comment(ehu_cat,"ESO QC SKYBRIGHT",
                                      "[mag/arcsec**2] sky brightness");
-        cpl_propertylist_update_double(ehu_cat,"PHOTZP",med3 + 2.5*log10(exptime) - extinct);
+        cpl_propertylist_update_double(ehu_cat,"PHOTZP",photzp);
         cpl_propertylist_set_comment(ehu_cat,"PHOTZP",
                                      "[mag] photometric zeropoint");
         cpl_propertylist_update_double(ehu_cat,"PHOTZPER",sig3det);
@@ -678,54 +693,59 @@ extern int casu_photcal_extinct(casu_fits **images, casu_tfits **mstds,
 
         /* Update the matched standards headers */
 
-	if (mstds[i] != NULL) {
-	    ehu_cat = casu_tfits_get_ehu(mstds[i]);
-	    cpl_propertylist_update_int(ehu_cat,"ESO DRS MAGNZPTALL",nresall);
-	    cpl_propertylist_set_comment(ehu_cat,"ESO DRS MAGNZPTALL",
-					 "number of stars in all magzpt calc");
-	    cpl_propertylist_update_double(ehu_cat,"ESO DRS ZPALL1",med3);
-	    cpl_propertylist_set_comment(ehu_cat,"ESO DRS ZPALL1",
-					 "[mag] zeropoint 1*rcore all group images");
-	    cpl_propertylist_update_double(ehu_cat,"ESO DRS ZPSIGALL1",sig3);
-	    cpl_propertylist_set_comment(ehu_cat,"ESO DRS ZPSIGALL1",
-					 "[mag] zeropoint sigma 1*rcore all group images");
-	    cpl_propertylist_update_double(ehu_cat,"ESO DRS ZPALL2",med5);
-	    cpl_propertylist_set_comment(ehu_cat,"ESO DRS ZPALL2",
-					 "[mag] zeropoint 2*rcore all group images");
-	    cpl_propertylist_update_double(ehu_cat,"ESO DRS ZPSIGALL2",sig5);
-	    cpl_propertylist_set_comment(ehu_cat,"ESO DRS ZPSIGALL2",
-					 "[mag] zeropoint sigma 2*rcore all group images");
-	    cpl_propertylist_update_double(ehu_cat,"ESO DRS MEDEBV",medebv);
-	    cpl_propertylist_set_comment(ehu_cat,"ESO DRS MEDEBV",
-					 "[mag] median galactic colour excess");
-	    cpl_propertylist_update_double(ehu_cat,"ESO DRS SIGDET1",sig3det);
-	    cpl_propertylist_set_comment(ehu_cat,"ESO DRS SIGDET1",
-					 "[mag] sigma det-level zpt 1*rcore");
-	    cpl_propertylist_update_double(ehu_cat,"ESO DRS SIGDET2",sig5det);
-	    cpl_propertylist_set_comment(ehu_cat,"ESO DRS SIGDET2",
-					 "[mag] sigma det-level zpt 2*rcore");
-	    cpl_propertylist_update_double(ehu_cat,"ESO QC MAGZPT",med3);
-	    cpl_propertylist_set_comment(ehu_cat,"ESO QC MAGZPT",
-					 "[mag] photometric zeropoint");
-	    cpl_propertylist_update_double(ehu_cat,"ESO QC MAGZERR",sig3det);
-	    cpl_propertylist_set_comment(ehu_cat,"ESO QC MAGZERR",
-					 "[mag] photometric zeropoint error");
-	    cpl_propertylist_update_double(ehu_cat,"ESO QC SKYBRIGHT",sumskbr);
-	    cpl_propertylist_set_comment(ehu_cat,"ESO QC SKYBRIGHT",
-					 "[mag/arcsec**2] sky brightness");
-	    cpl_propertylist_update_double(ehu_cat,"PHOTZP",med3 + 2.5*log10(exptime) - extinct);
-	    cpl_propertylist_set_comment(ehu_cat,"PHOTZP",
-					 "[mag] photometric zeropoint");
-	    cpl_propertylist_update_double(ehu_cat,"PHOTZPER",sig3det);
-	    cpl_propertylist_set_comment(ehu_cat,"PHOTZPER",
-					 "[mag] uncertainty on PHOTZP");
-	    cpl_propertylist_update_int(ehu_cat,"ESO QC MAGNZPT",nresimtot);
-	    cpl_propertylist_set_comment(ehu_cat,"ESO QC MAGNZPT",
-					 "number of stars in magzpt calc");
-	    cpl_propertylist_update_bool(ehu_cat,"ZPFUDGED",isrubbish);
-	    cpl_propertylist_set_comment(ehu_cat,"ZPFUDGED",
-					 "TRUE if the ZP not derived from stds");
-	}
+        if (mstds[i] != NULL) {
+            phu_cat = casu_tfits_get_phu(mstds[i]);
+            cpl_propertylist_update_string(phu_cat,"FLUXCAL",fluxcal);
+            cpl_propertylist_set_comment(phu_cat,"FLUXCAL",
+                                         "Certifies the validity of PHOTZP");
+
+            ehu_cat = casu_tfits_get_ehu(mstds[i]);
+            cpl_propertylist_update_int(ehu_cat,"ESO DRS MAGNZPTALL",nresall);
+            cpl_propertylist_set_comment(ehu_cat,"ESO DRS MAGNZPTALL",
+                                         "number of stars in all magzpt calc");
+            cpl_propertylist_update_double(ehu_cat,"ESO DRS ZPALL1",med3);
+            cpl_propertylist_set_comment(ehu_cat,"ESO DRS ZPALL1",
+                                         "[mag] zeropoint 1*rcore all group images");
+            cpl_propertylist_update_double(ehu_cat,"ESO DRS ZPSIGALL1",sig3);
+            cpl_propertylist_set_comment(ehu_cat,"ESO DRS ZPSIGALL1",
+                                         "[mag] zeropoint sigma 1*rcore all group images");
+            cpl_propertylist_update_double(ehu_cat,"ESO DRS ZPALL2",med5);
+            cpl_propertylist_set_comment(ehu_cat,"ESO DRS ZPALL2",
+                                         "[mag] zeropoint 2*rcore all group images");
+            cpl_propertylist_update_double(ehu_cat,"ESO DRS ZPSIGALL2",sig5);
+            cpl_propertylist_set_comment(ehu_cat,"ESO DRS ZPSIGALL2",
+                                         "[mag] zeropoint sigma 2*rcore all group images");
+            cpl_propertylist_update_double(ehu_cat,"ESO DRS MEDEBV",medebv);
+            cpl_propertylist_set_comment(ehu_cat,"ESO DRS MEDEBV",
+                                         "[mag] median galactic colour excess");
+            cpl_propertylist_update_double(ehu_cat,"ESO DRS SIGDET1",sig3det);
+            cpl_propertylist_set_comment(ehu_cat,"ESO DRS SIGDET1",
+                                         "[mag] sigma det-level zpt 1*rcore");
+            cpl_propertylist_update_double(ehu_cat,"ESO DRS SIGDET2",sig5det);
+            cpl_propertylist_set_comment(ehu_cat,"ESO DRS SIGDET2",
+                                         "[mag] sigma det-level zpt 2*rcore");
+            cpl_propertylist_update_double(ehu_cat,"ESO QC MAGZPT",med3);
+            cpl_propertylist_set_comment(ehu_cat,"ESO QC MAGZPT",
+                                         "[mag] photometric zeropoint");
+            cpl_propertylist_update_double(ehu_cat,"ESO QC MAGZERR",sig3det);
+            cpl_propertylist_set_comment(ehu_cat,"ESO QC MAGZERR",
+                                         "[mag] photometric zeropoint error");
+            cpl_propertylist_update_double(ehu_cat,"ESO QC SKYBRIGHT",sumskbr);
+            cpl_propertylist_set_comment(ehu_cat,"ESO QC SKYBRIGHT",
+                                         "[mag/arcsec**2] sky brightness");
+            cpl_propertylist_update_double(ehu_cat,"PHOTZP",photzp);
+            cpl_propertylist_set_comment(ehu_cat,"PHOTZP",
+                                         "[mag] photometric zeropoint");
+            cpl_propertylist_update_double(ehu_cat,"PHOTZPER",sig3det);
+            cpl_propertylist_set_comment(ehu_cat,"PHOTZPER",
+                                         "[mag] uncertainty on PHOTZP");
+            cpl_propertylist_update_int(ehu_cat,"ESO QC MAGNZPT",nresimtot);
+            cpl_propertylist_set_comment(ehu_cat,"ESO QC MAGNZPT",
+                                         "number of stars in magzpt calc");
+            cpl_propertylist_update_bool(ehu_cat,"ZPFUDGED",isrubbish);
+            cpl_propertylist_set_comment(ehu_cat,"ZPFUDGED",
+                                         "TRUE if the ZP not derived from stds");
+        }
     }
     freespace(dodgy);
 
@@ -737,60 +757,65 @@ extern int casu_photcal_extinct(casu_fits **images, casu_tfits **mstds,
 
 /*---------------------------------------------------------------------------*/
 /**
-	\par Name:
-		casu_remove_mag_inside_range
-	\par Purpose:
-		Remove from mstds the stars having magnitude between low_mag and high_mag
-	\par Description:
-		Remove from mstds the stars having magnitude between low_mag and high_mag
-	\par Language:
-		C
-	\param mstds
-		The matched standards tables list
-	\param nimages
-		Number of elements in the matched standards tables list
-	\param filt
-		The name of the filter for the observation
-	\param phottab
-		The relevant table for photometric calibration for a given standard
-		source
-	\param low_mag
-		The lower bound of the range to remove
-	\param high_mag
-		The upper bound of the range to remove
-	\return The number of removed stars, -1 in case of error
-	\author
-		Matteo Salmistraro, ESO
+    \par Name:
+        casu_remove_mag_outside_range
+    \par Purpose:
+        Remove from mstds the stars having magnitude outside low_mag and high_mag
+    \par Description:
+        Remove from mstds the stars having magnitude outside low_mag and high_mag
+    \par Language:
+        C
+    \param mstds
+        The matched standards tables list
+    \param nimages
+        Number of elements in the matched standards tables list
+    \param filt
+        The name of the filter for the observation
+    \param phottab
+        The relevant table for photometric calibration for a given standard
+        source
+    \param low_mag
+        The lower magnitude bound. Stars brigher than this are removed
+    \param high_mag
+        The upper magnitude bound. Stars fainter than this are removed
+    \return The number of removed stars, -1 in case of error
+    \author
+        Matteo Salmistraro, ESO
 */
 /*---------------------------------------------------------------------------*/
-extern int casu_remove_mag_inside_range(casu_tfits **mstds, const int nimages,
-										 char *filt, cpl_table *phottab,
-										 const float low_mag, const float high_mag)
+extern int casu_remove_mag_outside_range(casu_tfits **mstds, const int nimages,
+                                        char *filt, cpl_table *phottab,
+                                        const float low_mag, const float high_mag)
 {
 
-	photstrct p;
+    photstrct p;
 
-	if (casu_phot_open(phottab,filt,&p) != CASU_OK)
-		return -1;
+    if (casu_phot_open(phottab,filt,&p) != CASU_OK)
+        return -1;
 
-	int i, j;
-	int num_erased = 0;
-	for (i = 0; i < nimages; i++) {
-		 casu_tfits * ctfits = mstds[i];
-		 cpl_table * stds = casu_tfits_get_table(ctfits);
-		 cpl_table_unselect_all(stds);
-		 for (j = 0; j < p.ncolumns_coleq; j++) {
-			 const char * clnm_name = (p.coleq_columns)[j];
-			 cpl_table_and_selected_float(stds, clnm_name, CPL_GREATER_THAN, low_mag);
-			 cpl_table_and_selected_float(stds, clnm_name, CPL_LESS_THAN, high_mag);
-		 }
-		 num_erased += cpl_table_count_selected(stds);
-		 cpl_table_erase_selected(stds);
-		 cpl_table_select_all(stds);
-	}
+    int i, j;
+    int num_erased = 0;
+    for (i = 0; i < nimages; i++) {
+        casu_tfits * ctfits = mstds[i];
+        cpl_table * stds = casu_tfits_get_table(ctfits);
+        cpl_table_unselect_all(stds);
+        for (j = 0; j < p.ncolumns_coleq; j++) {
+            const char * clnm_name = (p.coleq_columns)[j];
+            if(!cpl_table_has_column(stds, clnm_name))
+            {
+                casu_phot_close(&p);
+                return -1;
+            }
+            cpl_table_or_selected_float(stds, clnm_name, CPL_LESS_THAN, low_mag);
+            cpl_table_or_selected_float(stds, clnm_name, CPL_GREATER_THAN, high_mag);
+        }
+        num_erased += cpl_table_count_selected(stds);
+        cpl_table_erase_selected(stds);
+        cpl_table_select_all(stds);
+    }
 
-	casu_phot_close(&p);
-	return num_erased;
+    casu_phot_close(&p);
+    return num_erased;
 }
 
 
@@ -1070,7 +1095,7 @@ static int extract_coleq(cpl_table *tab, photstrct *p) {
     /* Now parse the string into the colour equation values */
 
     z = cpl_malloc(nv*sizeof(float));
-    char *saveptr1;
+    char *saveptr1 = NULL;
     for (i = 0; i < nv; i++) {
         if (i == 0) 
             w = strtok_r(v,",", &saveptr1);
