@@ -37,11 +37,11 @@
 
 #include "casu_sky.h"
 #include "casu_mask.h"
-#include "casu_fits.h"
+#include "catalogue/casu_fits.h"
 #include "casu_mods.h"
-#include "casu_utils.h"
+#include "catalogue/casu_utils.h"
 #include "casu_stats.h"
-#include "casu_wcsutils.h"
+#include "catalogue/casu_wcsutils.h"
 
 static void casu_sky_mask_grow(cpl_image *dith, float rad);
 static float casu_sky_med(casu_fits *sky);
