@@ -32,8 +32,8 @@
 #include <math.h>
 
 #include <cpl.h>
-#include "../casu_fits.h"
-#include "../casu_wcsutils.h"
+#include "casu_fits.h"
+#include "casu_wcsutils.h"
 
 #include "ap.h"
 #include "util.h"
