@@ -504,8 +504,13 @@ int hawki_bkg_from_running_mean_frame_extension
     		 i_target - from_ima, half_width, rejlow, rejhigh, bkg);
     
     /* Free. The images in imagelist_window are unset but not deallocated */
-    for(i_ima=from_ima; i_ima<=to_ima ; i_ima++)
+    for(i_ima=from_ima; i_ima<=to_ima ; i_ima++) {
+        /* To remove a false positive detected by cppcheck*/
+
+        // cppcheck-suppress leakReturnValNotUsed
+
         cpl_imagelist_unset(imagelist_window, 0);
+    }
     cpl_imagelist_delete(imagelist_window);
     cpl_vector_delete(medians_window);
 
