@@ -126,8 +126,11 @@ extern int hawki_pfits_get_filter(const cpl_propertylist *plist, char *filt) {
 
     cpl_errorstate prestate = cpl_errorstate_get();
 
-    strncpy(filt1,cpl_propertylist_get_string(plist,"ESO INS FILT1 NAME"),16);
-    strncpy(filt2,cpl_propertylist_get_string(plist,"ESO INS FILT2 NAME"),16);
+    strncpy(filt1,cpl_propertylist_get_string(plist,"ESO INS FILT1 NAME"),16-1);
+    strncpy(filt2,cpl_propertylist_get_string(plist,"ESO INS FILT2 NAME"),16-1);
+    filt1[16-1] = '\0';
+    filt2[16-1] = '\0';
+
     if (cpl_errorstate_is_equal(prestate)) {
         if (! strncmp(filt1,"OPEN",16)) {
             strncpy(filt,filt2,16);
