@@ -38,8 +38,8 @@
 #include <math.h>
 #include <cpl.h>
 
-#include "hawki_dfs.h"
-#include "hawki_pfits.h"
+#include "hawki_dfs_legacy.h"
+#include "hawki_pfits_legacy.h"
 #include "hawki_properties_tel.h"
 
 /*----------------------------------------------------------------------------*/
