@@ -61,6 +61,10 @@
 
 /**@{*/
 
+static void
+hawki_propagate_table(cpl_size ext, const char* filename_from,
+                          const char* filename_to);
+
 /*---------------------------------------------------------------------------*/
 /**
     \par Name:
@@ -733,6 +737,9 @@ extern int hawki_testfrm_1(cpl_frame *fr, int nextn_expected, int isimg,
        if it matches the number expected */
 
     nextn = cpl_frame_get_nextensions(fr);
+
+    nextn = hawki_aodata_nextn_correct(nextn);
+
     if (nextn != nextn_expected) {
         cpl_msg_error(fctid,"Frame %s has %" CPL_SIZE_FORMAT " extensions, expected %" CPL_SIZE_FORMAT "\n",
                       cpl_frame_get_filename(fr),(cpl_size)nextn,
@@ -955,6 +962,126 @@ extern void hawki_copywcs(cpl_propertylist *p1, cpl_propertylist *p2) {
     cpl_propertylist_copy_property_regexp(p2,p1,CPL_WCS_REGEXP,0);
 }
 
+/*----------------------------------------------------------------------------*/
+/**
+  @brief    Correct the number of retrieved extensions for adaptive optic data
+  @param    nextn   Number of retrieved extensions
+  @return   corrected number of extensions for AO data
+ */
+/*----------------------------------------------------------------------------*/
+
+int hawki_aodata_nextn_correct(int nextn)
+{
+    if (nextn > HAWKI_NEXTN) {
+        /* Assuming data from adaptive optics are in the last extensions that
+         * should be ignored by the pipeline */
+        cpl_msg_info(cpl_func, "Number of extension exceeds %d, ignoring all"
+                        " extensions larger than %d", HAWKI_NEXTN, HAWKI_NEXTN);
+        return HAWKI_NEXTN;
+    }
+    else {
+        return nextn;
+    }
+}
+
+
+/*----------------------------------------------------------------------------*/
+/**
+  @brief    Propagates the adaptive optics extension from the raw to the product
+  @param    frameset  The frameset with the raw and product files
+  @param    from  Tag of the frames with the AO extesnions
+  @param    to Tag of the frames where the AO extensions should be copied
+  @return   CPL_ERROR_NONE if no real error happened.
+ */
+/*----------------------------------------------------------------------------*/
+
+cpl_error_code hawki_propagate_aoextensions(cpl_frameset * frameset,
+                                            const char * from, const char * to)
+{
+    /* Check input in not empty*/
+
+    cpl_error_ensure(frameset != NULL, CPL_ERROR_NULL_INPUT,
+                    return CPL_ERROR_NULL_INPUT, "Framelist is empty");
+    cpl_error_ensure(from != NULL, CPL_ERROR_NULL_INPUT,
+                     return CPL_ERROR_NULL_INPUT, "from string is pointing to NULL");
+    cpl_error_ensure(to != NULL, CPL_ERROR_NULL_INPUT,
+                     return CPL_ERROR_NULL_INPUT, "to string is pointing to NULL");
+
+
+    cpl_frameset * fs_from = cpl_frameset_new();
+    cpl_frameset * fs_to = cpl_frameset_new();
+
+    /* populate from and to framesets */
+    for (cpl_size i = 0; i < cpl_frameset_get_size(frameset); i++) {
+        cpl_frame * frm =
+                        cpl_frame_duplicate(cpl_frameset_get_position(frameset, i));
+        if (!strcmp(cpl_frame_get_tag(frm), from)) {
+            cpl_frameset_insert(fs_from, frm);
+        }
+        else if (!strcmp(cpl_frame_get_tag(frm), to)) {
+            cpl_frameset_insert(fs_to, frm);
+        }
+        else{
+            cpl_frame_delete(frm);
+        }
+    }
+
+    /* Check that the number of from and to are the same and larger then 0 */
+    if (cpl_frameset_get_size(fs_to) == 0 || cpl_frameset_get_size(fs_from) == 0 ||
+                    cpl_frameset_get_size(fs_to) != cpl_frameset_get_size(fs_from)) {
+        cpl_frameset_delete(fs_to);
+        cpl_frameset_delete(fs_from);
+
+        cpl_msg_info(cpl_func, "No AO extensions propagated");
+        return CPL_ERROR_NONE;
+    }
+
+    cpl_size ext = 0;
+    cpl_frame * frm_from = NULL;
+    cpl_frame * frm_to = NULL;
+    const char * filename_from = NULL;
+    const char * filename_to = NULL;
+
+    /* propagate the AO extensions */
+    for (cpl_size i = 0; i < cpl_frameset_get_size(fs_from); i++) {
+
+        frm_from = cpl_frameset_get_position(fs_from, i);
+        frm_to = cpl_frameset_get_position(fs_to, i);
+        filename_from = cpl_frame_get_filename(frm_from);
+        filename_to = cpl_frame_get_filename(frm_to);
+
+        ext = cpl_fits_find_extension(filename_from, "SPARTA_ATM_DATA");
+        hawki_propagate_table(ext, filename_from, filename_to);
+
+        ext = cpl_fits_find_extension(filename_from, "TCS_DATA");
+        hawki_propagate_table(ext, filename_from, filename_to);
+    }
+
+    cpl_frameset_delete(fs_to);
+    cpl_frameset_delete(fs_from);
+
+    return cpl_error_get_code();
+}
+
+static void
+hawki_propagate_table(cpl_size ext, const char* filename_from,
+                          const char* filename_to)
+{
+    if (ext > 0) {
+        cpl_propertylist* plist = cpl_propertylist_load(filename_from, ext);
+        /* only copy if it is a table */
+        const char* checktable = cpl_propertylist_get_string(plist, "XTENSION");
+        if (checktable != NULL && strcmp(checktable, "BINTABLE") == 0) {
+            cpl_table* tbl_from = cpl_table_load(filename_from, ext, 0);
+            cpl_table_save(tbl_from, NULL, plist, filename_to, CPL_IO_EXTEND);
+            cpl_table_delete(tbl_from);
+        }
+        cpl_propertylist_delete(plist);
+    }
+}
+
+
+
 /**@}*/
 
 /*
