@@ -25,15 +25,19 @@
  * $Name: not supported by cvs2svn $
  */
 
-#ifdef HAVE_CONFIG_H
-#include <config.h>
-#endif
-
 /*-----------------------------------------------------------------------------
                                    Includes
  -----------------------------------------------------------------------------*/
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include "irplib_polynomial.h"
+
+/* IRPLIB_SWAP_DOUBLE: */
+#include "irplib_utils.h"
+
 #include <assert.h>
 #include <math.h>
 /* DBL_MAX: */
@@ -48,18 +52,6 @@
 /*----------------------------------------------------------------------------*/
 /**@{*/
 
-/*-----------------------------------------------------------------------------
-                                   Macro definitions
- -----------------------------------------------------------------------------*/
-
-#define IRPLIB_SWAP(a,b) { const double t=(a);(a)=(b);(b)=t; }
-
-#if 0
-#define irplib_trace() cpl_msg_info(cpl_func, "%d: Trace", __LINE__)
-#else
-#define irplib_trace() /* Trace */
-#endif
-
 /*-----------------------------------------------------------------------------
                                    Static functions
  -----------------------------------------------------------------------------*/
@@ -97,14 +89,24 @@ static cpl_error_code irplib_polynomial_solve_1d_4(double, double, double,
                                                    double *, double *,
                                                    double *, double *);
 
+static cpl_error_code irplib_polynomial_solve_1d_zero(cpl_polynomial *,
+                                                      cpl_vector *,
+                                                      cpl_size *)
+    CPL_ATTR_NONNULL;
+
 static cpl_error_code irplib_polynomial_solve_1d_nonzero(cpl_polynomial *,
                                                          cpl_vector *,
-                                                         cpl_size *);
+                                                         cpl_size *)
+    CPL_ATTR_NONNULL;
 
 static cpl_error_code irplib_polynomial_divide_1d_root(cpl_polynomial *, double,
                                                        double *);
 
-#ifdef IPRLIB_POLYNOMIAL_USE_MONOMIAL_ROOT
+static cpl_error_code irplib_polynomial_solve_1d_guess(const cpl_polynomial *,
+                                                       double *)
+    CPL_ATTR_NONNULL;
+
+#ifdef IRPLIB_POLYNOMIAL_GUESS_ANASOL
 static double irplib_polynomial_depress_1d(cpl_polynomial *);
 #endif
 
@@ -112,174 +114,6 @@ static double irplib_polynomial_depress_1d(cpl_polynomial *);
                               Function codes
  -----------------------------------------------------------------------------*/
 
-#if defined CPL_VERSION_CODE && CPL_VERSION_CODE > CPL_VERSION(5, 92, 0)
-#else
-
-/*----------------------------------------------------------------------------*/
-/**
-  @brief    Add two polynomials of the same dimension
-  @param    self      The polynomial to hold the result
-  @param    first     The 1st polynomial to add
-  @param    second    The 2nd polynomial to add
-  @return   CPL_ERROR_NONE or the relevant CPL error code
-  @note self may be passed also as first and/or second
-
-  Possible CPL error code set in this function:
-  - CPL_ERROR_NULL_INPUT if an input pointer is NULL
-  - CPL_ERROR_INCOMPATIBLE_INPUT if the polynomials do not have identical
-    dimensions
-  - CPL_ERROR_UNSUPPORTED_MODE if the dimension is not 1 (FIXME)
- */
-/*----------------------------------------------------------------------------*/
-cpl_error_code irplib_polynomial_add(cpl_polynomial * self,
-                                     const cpl_polynomial * first,
-                                     const cpl_polynomial * second)
-{
-    cpl_size       degree0 = cpl_polynomial_get_degree(self);
-    const cpl_size degree1 = cpl_polynomial_get_degree(first);
-    const cpl_size degree2 = cpl_polynomial_get_degree(second);
-    const cpl_size maxdeg  = degree1 > degree2 ? degree1 : degree2;
-
-
-    cpl_ensure_code(self   != NULL, CPL_ERROR_NULL_INPUT);
-    cpl_ensure_code(first  != NULL, CPL_ERROR_NULL_INPUT);
-    cpl_ensure_code(second != NULL, CPL_ERROR_NULL_INPUT);
-
-    cpl_ensure_code(cpl_polynomial_get_dimension(self) ==
-                    cpl_polynomial_get_dimension(first),
-                    CPL_ERROR_INCOMPATIBLE_INPUT);
-    cpl_ensure_code(cpl_polynomial_get_dimension(self) ==
-                    cpl_polynomial_get_dimension(second),
-                    CPL_ERROR_INCOMPATIBLE_INPUT);
-
-    /* FIXME: */
-    cpl_ensure_code(cpl_polynomial_get_dimension(self) == 1,
-                    CPL_ERROR_UNSUPPORTED_MODE);
-
-    if (degree0 < maxdeg) {
-        degree0 = maxdeg;
-    } else {
-        /* Reset coefficients in self as needed */
-        for (; degree0 > maxdeg; degree0--) {
-            cpl_polynomial_set_coeff(self, &degree0, 0.0);
-        }
-    }
-
-    /* assert( degree0 == maxdeg ); */
-
-    for (; degree0 >= 0; degree0--) {
-        const double val1 = cpl_polynomial_get_coeff(first, &degree0);
-        const double val2 = cpl_polynomial_get_coeff(second, &degree0);
-        cpl_polynomial_set_coeff(self, &degree0, val1 + val2);
-    }
-
-    return CPL_ERROR_NONE;
-}
-
-/*----------------------------------------------------------------------------*/
-/**
-  @brief    Subtract two polynomials of the same dimension
-  @param    self      The polynomial to hold the result
-  @param    first     The polynomial to subtract from
-  @param    second    The polynomial to subtract
-  @return   CPL_ERROR_NONE or the relevant CPL error code
-  @note self may be passed also as first and/or second
-
-  Possible CPL error code set in this function:
-  - CPL_ERROR_NULL_INPUT if an input pointer is NULL
-  - CPL_ERROR_INCOMPATIBLE_INPUT if the polynomials do not have identical
-    dimensions
-  - CPL_ERROR_UNSUPPORTED_MODE if the dimension is not 1 (FIXME)
- */
-/*----------------------------------------------------------------------------*/
-cpl_error_code irplib_polynomial_subtract(cpl_polynomial * self,
-                                          const cpl_polynomial * first,
-                                          const cpl_polynomial * second)
-{
-    cpl_size       degree0 = cpl_polynomial_get_degree(self);
-    const cpl_size degree1 = cpl_polynomial_get_degree(first);
-    const cpl_size degree2 = cpl_polynomial_get_degree(second);
-    const cpl_size maxdeg  = degree1 > degree2 ? degree1 : degree2;
-
-
-    cpl_ensure_code(self   != NULL, CPL_ERROR_NULL_INPUT);
-    cpl_ensure_code(first  != NULL, CPL_ERROR_NULL_INPUT);
-    cpl_ensure_code(second != NULL, CPL_ERROR_NULL_INPUT);
-
-    cpl_ensure_code(cpl_polynomial_get_dimension(self) ==
-                    cpl_polynomial_get_dimension(first),
-                    CPL_ERROR_INCOMPATIBLE_INPUT);
-    cpl_ensure_code(cpl_polynomial_get_dimension(self) ==
-                    cpl_polynomial_get_dimension(second),
-                    CPL_ERROR_INCOMPATIBLE_INPUT);
-
-    /* FIXME: */
-    cpl_ensure_code(cpl_polynomial_get_dimension(self) == 1,
-                    CPL_ERROR_UNSUPPORTED_MODE);
-
-    if (degree0 < maxdeg) {
-        degree0 = maxdeg;
-    } else {
-        /* Reset coefficients in self as needed */
-        for (; degree0 > maxdeg; degree0--) {
-            cpl_polynomial_set_coeff(self, &degree0, 0.0);
-        }
-    }
-
-    /* assert( degree0 == maxdeg ); */
-
-    for (; degree0 >= 0; degree0--) {
-        const double val1 = cpl_polynomial_get_coeff(first, &degree0);
-        const double val2 = cpl_polynomial_get_coeff(second, &degree0);
-        cpl_polynomial_set_coeff(self, &degree0, val1 - val2);
-    }
-
-    return CPL_ERROR_NONE;
-}
-
-/*----------------------------------------------------------------------------*/
-/**
-  @brief    Multiply a polynomial with a scalar
-  @param    self      The polynomial to hold the result
-  @param    other     The polynomial to scale, may equal self
-  @param    factor    The factor to multiply with
-  @return   CPL_ERROR_NONE or the relevant CPL error code
-
-  Possible CPL error code set in this function:
-  - CPL_ERROR_NULL_INPUT if an input pointer is NULL
-  - CPL_ERROR_UNSUPPORTED_MODE if the dimension is not 1 (FIXME)
- */
-/*----------------------------------------------------------------------------*/
-cpl_error_code irplib_polynomial_multiply_scalar(cpl_polynomial * self,
-                                                 const cpl_polynomial * other,
-                                                 double factor)
-{
-
-    const cpl_size maxdeg  = cpl_polynomial_get_degree(other);
-    const cpl_size zerodeg = cpl_polynomial_get_degree(self);
-    cpl_size       degree;
-
-    cpl_ensure_code(self  != NULL, CPL_ERROR_NULL_INPUT);
-    cpl_ensure_code(other != NULL, CPL_ERROR_NULL_INPUT);
-
-    cpl_ensure_code(cpl_polynomial_get_dimension(self) == 1,
-                    CPL_ERROR_UNSUPPORTED_MODE);
-    cpl_ensure_code(cpl_polynomial_get_dimension(other) == 1,
-                    CPL_ERROR_UNSUPPORTED_MODE);
-
-    for (degree = 0; degree <= maxdeg; degree++) {
-        const double val = factor * cpl_polynomial_get_coeff(other, &degree);
-        cpl_polynomial_set_coeff(self, &degree, val);
-    }
-
-    /* Reset coefficients in self as needed */
-    for (; degree <= zerodeg; degree++) {
-        cpl_polynomial_set_coeff(self, &zerodeg, 0.0);
-    }
-
-    return CPL_ERROR_NONE;
-}
-#endif
 /*----------------------------------------------------------------------------*/
 /**
   @brief    Compute all n roots of p(x) = 0, where p(x) is of degree n, n > 0.
@@ -287,66 +121,7 @@ cpl_error_code irplib_polynomial_multiply_scalar(cpl_polynomial * self,
   @param    roots A pre-allocated vector of length n to hold the roots
   @param    preal The number of real roots found, or undefined on error
   @return   CPL_ERROR_NONE or the relevant CPL error code
-
-  The *preal real roots are stored first in ascending order, then follows for
-  each pair of complex conjugate roots, the real and imaginary parts of the
-  root in the positive imaginary half-plane, for example for a 3rd degree
-  polynomial with 1 real root, the roots are represented as:
-  x0 = v0
-  x1 = v1 + i v2
-  x2 = v1 - i v2,
-  where v0, v1, v2 are the elements of the roots vector.
-
-  Possible CPL error code set in this function:
-  - CPL_ERROR_NULL_INPUT if an input pointer is NULL
-  - CPL_ERROR_INVALID_TYPE if the polynomial has the wrong dimension
-  - CPL_ERROR_DATA_NOT_FOUND if the polynomial does not have a degree of at
-                             least 1.
-  - CPL_ERROR_INCOMPATIBLE_INPUT if the roots vector does not have length n
-  - CPL_ERROR_DIVISION_BY_ZERO if a division by zero occurs (n > 4)
-  - CPL_ERROR_CONTINUE if the algorithm does not converge (n > 4)
- */
-/*----------------------------------------------------------------------------*/
-cpl_error_code irplib_polynomial_solve_1d_all(const cpl_polynomial * self,
-                                              cpl_vector * roots,
-                                              cpl_size * preal)
-{
-
-    cpl_error_code error = CPL_ERROR_NONE;
-    cpl_polynomial * p;
-
-    cpl_ensure_code(self  != NULL, CPL_ERROR_NULL_INPUT);
-    cpl_ensure_code(roots != NULL, CPL_ERROR_NULL_INPUT);
-    cpl_ensure_code(preal != NULL, CPL_ERROR_NULL_INPUT);
-    cpl_ensure_code(cpl_polynomial_get_dimension(self) == 1,
-                    CPL_ERROR_INVALID_TYPE);
-    cpl_ensure_code(cpl_polynomial_get_degree(self) > 0,
-                    CPL_ERROR_DATA_NOT_FOUND);
-    cpl_ensure_code(cpl_polynomial_get_degree(self) ==
-                    cpl_vector_get_size(roots), CPL_ERROR_INCOMPATIBLE_INPUT);
-
-    *preal = 0;
-
-    p = cpl_polynomial_duplicate(self);
-
-    error = irplib_polynomial_solve_1d_nonzero(p, roots, preal);
-
-    cpl_polynomial_delete(p);
-
-    return error;
-
-}
-
-/**@}*/
-
-/*----------------------------------------------------------------------------*/
-/**
-  @internal
-  @brief    Compute all n roots of p(x) = 0, where p(x) is of degree n, n > 0.
-  @param    self  The 1D-polynomial
-  @param    roots A pre-allocated vector of length n to hold the roots
-  @param    preal The number of real roots found, or undefined on error
-  @return   CPL_ERROR_NONE or the relevant CPL error code
+  @note     There is (currently) no support for more than 4 complex roots
 
   The *preal real roots are stored first in ascending order, then follows for
   each pair of complex conjugate roots, the real and imaginary parts of the
@@ -364,6 +139,116 @@ cpl_error_code irplib_polynomial_solve_1d_all(const cpl_polynomial * self,
   - CPL_ERROR_INCOMPATIBLE_INPUT if the roots vector does not have length n
   - CPL_ERROR_DIVISION_BY_ZERO if a division by zero occurs (n > 4)
   - CPL_ERROR_CONTINUE if the algorithm does not converge (n > 4)
+ */
+/*----------------------------------------------------------------------------*/
+cpl_error_code irplib_polynomial_solve_1d_all(const cpl_polynomial * self,
+                                              cpl_vector * roots,
+                                              cpl_size * preal)
+{
+
+    cpl_error_code error;
+    const cpl_size degree = cpl_polynomial_get_degree(self);
+    cpl_polynomial * p;
+
+    cpl_ensure_code(self  != NULL, CPL_ERROR_NULL_INPUT);
+    cpl_ensure_code(roots != NULL, CPL_ERROR_NULL_INPUT);
+    cpl_ensure_code(preal != NULL, CPL_ERROR_NULL_INPUT);
+    cpl_ensure_code(cpl_polynomial_get_dimension(self) == 1,
+                    CPL_ERROR_INVALID_TYPE);
+    cpl_ensure_code(degree > 0,    CPL_ERROR_DATA_NOT_FOUND);
+    cpl_ensure_code(degree == cpl_vector_get_size(roots),
+                    CPL_ERROR_INCOMPATIBLE_INPUT);
+
+    *preal = 0;
+
+    p = cpl_polynomial_duplicate(self);
+
+    error = irplib_polynomial_solve_1d_zero(p, roots, preal);
+
+    if (!error && *preal < degree) {
+        /* There are non-zero roots */
+
+        /* Whether roots need sorting (no need w. up to 4 non-zero roots) */
+        const cpl_boolean dosort = *preal > 0 || degree - *preal > 4;
+
+        assert(cpl_polynomial_get_degree(p) + *preal == degree);
+
+        error = irplib_polynomial_solve_1d_nonzero(p, roots, preal);
+
+        if (!error && dosort) {
+            cpl_vector * reals = cpl_vector_wrap(*preal,
+                                                 cpl_vector_get_data(roots));
+            cpl_vector_sort(reals, CPL_SORT_ASCENDING);
+            (void)cpl_vector_unwrap(reals);
+        }
+    }
+
+    cpl_polynomial_delete(p);
+
+    return error ? cpl_error_set_where(cpl_func) : CPL_ERROR_NONE;
+
+}
+
+/*----------------------------------------------------------------------------*/
+/**
+  @internal
+  @brief    Divide out all zero-valued roots from the 1D-polynomial
+  @param    self  The 1D-polynomial
+  @param    roots A pre-allocated vector of length n to hold the roots
+  @param    preal The number of real roots found so far or undefined on error
+  @return   CPL_ERROR_NONE or the relevant CPL error code
+  @see      irplib_polynomial_solve_1d_all()
+  @note The caller may pass a roots vector which already has *preal (other)
+        roots which are only accessed when all roots are sorted at the end
+
+*/
+/*----------------------------------------------------------------------------*/
+static cpl_error_code irplib_polynomial_solve_1d_zero(cpl_polynomial * self,
+                                                      cpl_vector * roots,
+                                                      cpl_size * preal)
+{
+    cpl_size nzero;
+    const cpl_size degree = cpl_polynomial_get_degree(self);
+
+    /* Count number of zero-value roots */
+    for (nzero = 0; nzero < degree; nzero++) {
+        if (cpl_polynomial_get_coeff(self, &nzero) != 0.0) break;
+    }
+
+    if (nzero > 0) {
+        cpl_size i = 0;
+        for (; i <= degree - nzero; i++) {
+            const cpl_size icopy = i + nzero;
+            const double value = cpl_polynomial_get_coeff(self, &icopy);
+
+            if (cpl_polynomial_set_coeff(self, &i, value))
+                return cpl_error_set_where(cpl_func);
+        }
+        for (; i <= degree; i++) {
+            if (cpl_polynomial_set_coeff(self, &i, 0.0))
+                return cpl_error_set_where(cpl_func);
+            cpl_vector_set(roots, (*preal)++, 0.0);
+        }
+    }
+ 
+    return CPL_ERROR_NONE;
+
+}
+
+/**@}*/
+
+/*----------------------------------------------------------------------------*/
+/**
+  @internal
+  @brief    Compute all n roots of p(x) = 0, where p(x) is of degree n, n > 0
+  @param    self  The 1D-polynomial, constant term is non-zero
+  @param    roots A pre-allocated vector of length n to hold the roots
+  @param    preal The number of real roots found so far or undefined on error
+  @return   CPL_ERROR_NONE or the relevant CPL error code
+  @see      irplib_polynomial_solve_1d_all()
+  @note The caller may pass a roots vector which already has *preal (other)
+        roots which are only accessed when all roots are sorted at the end
+
 */
 /*----------------------------------------------------------------------------*/
 static cpl_error_code irplib_polynomial_solve_1d_nonzero(cpl_polynomial * self,
@@ -373,11 +258,7 @@ static cpl_error_code irplib_polynomial_solve_1d_nonzero(cpl_polynomial * self,
     cpl_error_code error   = CPL_ERROR_NONE;
     const cpl_size ncoeffs = 1 + cpl_polynomial_get_degree(self);
 
-    cpl_ensure_code(self  != NULL,  CPL_ERROR_NULL_INPUT);
-    cpl_ensure_code(roots != NULL,  CPL_ERROR_NULL_INPUT);
-    cpl_ensure_code(preal != NULL,  CPL_ERROR_NULL_INPUT);
-    cpl_ensure_code(cpl_polynomial_get_dimension(self) == 1,
-                    CPL_ERROR_INVALID_TYPE);
+
     cpl_ensure_code(ncoeffs   > 1,  CPL_ERROR_DATA_NOT_FOUND);
     cpl_ensure_code(*preal >= 0,    CPL_ERROR_ILLEGAL_INPUT);
     cpl_ensure_code(ncoeffs + *preal == 1+cpl_vector_get_size(roots),
@@ -391,6 +272,8 @@ static cpl_error_code irplib_polynomial_solve_1d_nonzero(cpl_polynomial * self,
         const double   p1 = cpl_polynomial_get_coeff(self, &i1);
         const double   p0 = cpl_polynomial_get_coeff(self, &i0);
 
+        assert( p1 != 0.0 );
+
         cpl_vector_set(roots, (*preal)++, -p0/p1);
         break;
     }
@@ -403,6 +286,8 @@ static cpl_error_code irplib_polynomial_solve_1d_nonzero(cpl_polynomial * self,
         const double   p0 = cpl_polynomial_get_coeff(self, &i0);
         double         x1, x2;
 
+        assert( p2 != 0.0 );
+
         if (irplib_polynomial_solve_1d_2(p2, p1, p0, &x1, &x2)) {
             /* This is the complex root in the upper imaginary half-plane */
             cpl_vector_set(roots, (*preal)  , x1);
@@ -424,6 +309,8 @@ static cpl_error_code irplib_polynomial_solve_1d_nonzero(cpl_polynomial * self,
         const double   p0 = cpl_polynomial_get_coeff(self, &i0);
         double         x1, x2, x3;
 
+        assert( p3 != 0.0 );
+
         if (irplib_polynomial_solve_1d_3(p3, p2, p1, p0, &x1, &x2, &x3,
                                          NULL, NULL)) {
             cpl_vector_set(roots, (*preal)++, x1);
@@ -451,6 +338,8 @@ static cpl_error_code irplib_polynomial_solve_1d_nonzero(cpl_polynomial * self,
         double         x1, x2, x3, x4;
         cpl_size       nreal;
 
+        assert( p4 != 0.0 );
+
         error = irplib_polynomial_solve_1d_4(p4, p3, p2, p1, p0, &nreal,
                                              &x1, &x2, &x3, &x4);
         if (!error) {
@@ -467,28 +356,9 @@ static cpl_error_code irplib_polynomial_solve_1d_nonzero(cpl_polynomial * self,
     default: {
 
         /* Try to reduce the problem by finding a single root */
-#ifndef IPRLIB_POLYNOMIAL_USE_MONOMIAL_ROOT
-        const cpl_size    n0 = ncoeffs-1;
-        const double pn0 = cpl_polynomial_get_coeff(self, &n0);
-        const cpl_size    n1 = ncoeffs-2;
-        const double pn1 = cpl_polynomial_get_coeff(self, &n1);
-        /* First guess of root is the root average.
-           FIXME: May need refinement, e.g. via bisection */
-        const double rmean = -pn1 / (pn0 * n0);
-        double root = rmean;
-#else
-        /* Try an analytical solution to a (shifted) monomial */
-        cpl_polynomial * copy = cpl_polynomial_duplicate(self);
-        const cpl_size    i0 = 0;
-        const double rmean = irplib_polynomial_depress_1d(copy);
-        const double c0 = cpl_polynomial_get_coeff(copy, &i0);
-        double root = rmean + ((n0&1) && c0 < 0.0 ? -1.0 : 1.0)
-            * pow(fabs(c0), 1.0/n0);
+        double         root = 0.0;
 
-        cpl_polynomial_delete(copy);
-#endif
-
-        error = cpl_polynomial_solve_1d(self, root, &root, 1);
+        error = irplib_polynomial_solve_1d_guess(self, &root);
 
         if (!error) {
 
@@ -498,22 +368,170 @@ static cpl_error_code irplib_polynomial_solve_1d_nonzero(cpl_polynomial * self,
 
             error = irplib_polynomial_solve_1d_nonzero(self, roots, preal);
 
-            if (!error && *preal > 1) {
-                /* Sort the real roots */
-
-                /* FIXME: Assumes that all roots found so far are real */
-
-                cpl_vector * reals = cpl_vector_wrap(*preal,
-                                                     cpl_vector_get_data(roots));
-                cpl_vector_sort(reals, 1);
-                (void)cpl_vector_unwrap(reals);
-            }
         }
+
         break;
     }
     }
 
-    return error;
+    return error ? cpl_error_set_where(cpl_func) : CPL_ERROR_NONE;
+}
+
+/*----------------------------------------------------------------------------*/
+/**
+  @internal
+  @brief    Try to find a single, real root of the provided 1D-polynomial
+  @param    self  The 1D-polynomial, constant term is non-zero
+  @param    proot The root or undefined on error
+  @return   CPL_ERROR_NONE or the relevant CPL error code
+  @see      irplib_polynomial_solve_1d_all()
+
+*/
+/*----------------------------------------------------------------------------*/
+static
+cpl_error_code irplib_polynomial_solve_1d_guess(const cpl_polynomial * self,
+                                                double * proot)
+{
+    cpl_errorstate prestate = cpl_errorstate_get();
+    cpl_error_code error = CPL_ERROR_NONE;
+    const cpl_size degree = cpl_polynomial_get_degree(self);
+    const cpl_size ncand = 5; /* 2 is enough for current guessing strategy */
+    double rcand[ncand];
+    cpl_size icand = 0;
+    size_t ipos = 0, ineg = 0;
+    double rpos[1], rneg[1];
+    cpl_boolean do_bisect = CPL_FALSE;
+
+
+    /* If the derivative at the first guess happens to be zero, then
+       the first guess is no good, so try a few different ones. */
+
+    for (cpl_size itry = 0; ; itry++) {
+        switch (itry) {
+          case 0: {
+              /* Try the arithmetic mean of the roots */
+              const double   pn0   = cpl_polynomial_get_coeff(self, &degree);
+              const cpl_size n1    = degree-1;
+              const double   pn1   = cpl_polynomial_get_coeff(self, &n1);
+              double         rmean;
+
+              assert( pn0 != 0.0 );
+
+              rmean = -pn1 / (pn0 * (double)degree);
+
+              rcand[icand++] = rmean;
+
+              break;
+          }
+
+          case 1: {
+              /* Try the geometric mean of the roots */
+              const cpl_size i0 = 0;
+              const double   c0 = cpl_polynomial_get_coeff(self, &i0);
+              double         rmean;
+
+              assert( c0 != 0.0 );
+
+              rmean = pow(fabs(c0), 1.0/(double)degree);
+
+              rcand[icand++] = rmean;
+
+              break;
+          }
+
+          case 2: {
+              /* Try to get starting guesses with opposite sign residuals */
+              const cpl_size i0 = 0;
+              const double   c0 = cpl_polynomial_get_coeff(self, &i0);
+              const double   pn0   = cpl_polynomial_get_coeff(self, &degree);
+              const cpl_size n1    = degree-1;
+              const double   pn1   = cpl_polynomial_get_coeff(self, &n1);
+              double         rmean;
+
+              assert( pn0 != 0.0 );
+
+              rmean = -pn1 / (pn0 * (double)degree);
+
+              rcand[icand++] = rmean + c0;
+              rcand[icand++] = rmean - c0;
+
+              break;
+          }
+
+#ifdef IRPLIB_POLYNOMIAL_GUESS_ANASOL
+          case 3: {
+
+              /* Try an analytical solution to a (shifted) monomial */
+              cpl_polynomial * copy = cpl_polynomial_duplicate(self);
+              const cpl_size   i0   = 0;
+              const double rmean = irplib_polynomial_depress_1d(copy);
+              const double c0 = cpl_polynomial_get_coeff(copy, &i0);
+              const double radius = pow(fabs(c0), 1.0/(double)degree);
+
+              rcand[icand++] = rmean + radius;
+              if (radius != 0.0) /* Should always be true */
+                  rcand[icand++] = rmean - radius;
+
+              cpl_polynomial_delete(copy);
+
+              break;
+          }
+#endif
+
+        default:
+            /* From here on only first guesses increasingly refined via
+               bisection are tried */
+            if (ipos > 0 && ineg > 0) {
+                rcand[icand++] = 0.5 * (rpos[0] + rneg[0]);
+                do_bisect = CPL_TRUE;
+            }
+            break;
+        }
+
+        if (icand > 0) {
+            double grad;
+            double root = rcand[--icand];
+            const double resid = cpl_polynomial_eval_1d(self, root, &grad);
+
+#ifdef IRPLIB_POLYNOMIAL_DEBUG
+            if (itry > 0)
+                cpl_msg_warning(cpl_func, "RETRY(%d)=%g, degree=%d, r=%g, d=%g",
+                                (int)itry, root, (int)degree, resid, grad);
+#endif
+
+            error = cpl_polynomial_solve_1d(self, root, proot, 1);
+            if (!error) {
+                cpl_errorstate_set(prestate);
+                break;
+            }
+
+            if (do_bisect) {
+                *(resid > 0.0 ? rpos : rneg) = root;
+            } else {
+                /* Try to collect first guess with residuals with opposite signs */
+                if (resid > 0.0) {
+                    if (ipos == 0) rpos[ipos++] = root;
+                } else if (ineg == 0) {
+                    rneg[ineg++] = root;
+                }
+
+                if (ipos == 0 || ineg == 0) {
+                    const double resid2 = cpl_polynomial_eval_1d(self, *proot,
+                                                                 &grad);
+                    if (resid2 > 0.0) {
+                        if (ipos == 0) rpos[ipos++] = root;
+                    } else if (ineg == 0) {
+                        rneg[ineg++] = root;
+                    }
+                }
+            }
+
+        } else {
+            break;
+        }
+    }
+
+    return error ? cpl_error_set_where(cpl_func) : CPL_ERROR_NONE;
 }
 
 /*----------------------------------------------------------------------------*/
@@ -523,9 +541,11 @@ static cpl_error_code irplib_polynomial_solve_1d_nonzero(cpl_polynomial * self,
   @param    p2    The non-zero coefficient to x^2
   @param    p1    The coefficient to x
   @param    p0    The constant term
-  @param    px1   The 1st root or the real part of the complex roots
-  @param    px2   The 2nd root or the imaginary part of the complex roots
-  @return   CPL_TRUE iff the roots are complex
+  @param    px1   The 1st real root or the real part of the complex, conjugate
+                  roots
+  @param    px2   The 2nd real root or the positive, imaginary part of the
+                  complex roots
+  @return   CPL_TRUE iff the roots are complex conjugate
 
  */
 /*----------------------------------------------------------------------------*/
@@ -533,7 +553,9 @@ static cpl_boolean irplib_polynomial_solve_1d_2(double p2, double p1, double p0,
                                                 double * px1,
                                                 double * px2) {
 
-    const double sqrtD = sqrt(fabs(p1 * p1 - 4.0 * p2 * p0));
+    const double sqrtD = sqrt(p1 * p1 < 4.0 * p2 * p0
+                              ? 4.0 * p2 * p0 - p1 * p1
+                              : p1 * p1 - 4.0 * p2 * p0);
     cpl_boolean is_complex = CPL_FALSE;
     double x1 = -0.5 * p1 / p2; /* Double root */
     double x2;
@@ -552,10 +574,8 @@ static cpl_boolean irplib_polynomial_solve_1d_2(double p2, double p1, double p0,
     /* x1 is the root with largest absolute value */
     if (p1 > 0.0) {
         x1 = -0.5 * (p1 + sqrtD);
-        irplib_trace(); /* OK */
     } else {
         x1 = -0.5 * (p1 - sqrtD);
-        irplib_trace(); /* OK */
     }
     /* Compute smaller root via division to avoid
        loss of precision due to cancellation */
@@ -569,11 +589,9 @@ static cpl_boolean irplib_polynomial_solve_1d_2(double p2, double p1, double p0,
         if (x2 > x1) {
             *px1 = x1;
             *px2 = x2;
-            irplib_trace(); /* OK */
         } else {
             *px1 = x2;
             *px2 = x1;
-            irplib_trace(); /* OK */
         }
     }
 
@@ -588,7 +606,6 @@ static cpl_boolean irplib_polynomial_solve_1d_2(double p2, double p1, double p0,
         *px1 = x1;
         *px2 = x2;
         is_complex = CPL_TRUE;
-        irplib_trace(); /* OK */
     }
 
     return is_complex;
@@ -618,13 +635,11 @@ static double irplib_polynomial_eval_2_max(double p2, double p1, double p0,
 
     if (is_c) {
         res = fabs(p0 + x1 * (p1 + x1 * p2) - p2 * x2 * x2);
-        irplib_trace(); /* OK */
     } else {
         const double r1 = fabs(p0 + x1 * (p1 + x1 * p2));
         const double r2 = fabs(p0 + x2 * (p1 + x2 * p2));
 
         res = r1 > r2 ? r1 : r2;
-        irplib_trace(); /* OK */
     }
 
     return res;
@@ -660,12 +675,10 @@ static double irplib_polynomial_eval_3_max(double p3, double p2,
                                - x3 * x3 * ( 3.0 * p3 * x2 + p2));
 
         res = r1 > r2 ? r1 : r2;
-        irplib_trace(); /* OK */
     } else {
         const double r2 = fabs(p0 + x2 * (p1 + x2 * (p2 + x2 * p3)));
         const double r3 = fabs(p0 + x3 * (p1 + x3 * (p2 + x3 * p3)));
         res = r1 > r2 ? (r1 > r3 ? r1 : r3) : (r2 > r3 ? r2 : r3);
-        irplib_trace(); /* OK */
     }
 
     /* cpl_msg_info(cpl_func, "%d: %g (%g)", __LINE__, res, r1); */
@@ -760,7 +773,6 @@ static cpl_boolean irplib_polynomial_solve_1d_3(double p3, double p2, double p1,
         if (pdbl1 != NULL) *pdbl1 = CPL_FALSE;
         if (!is_c && pdbl2 != NULL) *pdbl2 = dbl2;
         is_complex = is_c;
-        irplib_trace(); /* OK */
    
     }
 
@@ -785,7 +797,6 @@ static cpl_boolean irplib_polynomial_solve_1d_3(double p3, double p2, double p1,
             if (pdbl1 != NULL) *pdbl1 = CPL_FALSE;
             if (pdbl2 != NULL) *pdbl2 = CPL_FALSE;
             is_complex = CPL_FALSE;
-            irplib_trace(); /* OK */
         }
     }
 
@@ -811,7 +822,6 @@ static cpl_boolean irplib_polynomial_solve_1d_3(double p3, double p2, double p1,
             if (pdbl1 != NULL) *pdbl1 = CPL_FALSE;
             if (pdbl2 != NULL) *pdbl2 = dbl2;
             is_complex = CPL_FALSE;
-            irplib_trace(); /* OK */
         }
 
 
@@ -825,14 +835,13 @@ static cpl_boolean irplib_polynomial_solve_1d_3(double p3, double p2, double p1,
 
         if (resx <= res) {
             is_first = CPL_FALSE;
-            res = resx;
+            /*res = resx;*/
             x1 = xx1;
             x2 = xx2;
             x3 = xx3;
             if (pdbl1 != NULL) *pdbl1 = dbl1;
             if (pdbl2 != NULL) *pdbl2 = CPL_FALSE;
             is_complex = CPL_FALSE;
-            irplib_trace(); /* OK */
         }
 
     }
@@ -841,13 +850,10 @@ static cpl_boolean irplib_polynomial_solve_1d_3(double p3, double p2, double p1,
         *px1 = x1;
         *px2 = x2;
         *px3 = x3;
-        irplib_trace(); /* OK */
     } else if (is_complex) {
         *px1 = x1;
-        irplib_trace(); /* OK */
     } else {
         *px1 = x3;
-        irplib_trace(); /* OK */
     }
 
     return is_complex;
@@ -885,7 +891,6 @@ static void irplib_polynomial_solve_1d_31(double a, double Q,
     *px2 = x2;
     *px3 = x3;
 
-    irplib_trace(); /* OK */
     return;
 }
 
@@ -923,18 +928,15 @@ static void irplib_polynomial_solve_1d_32(double a, double c, double Q,
            end up here, with the sign lost for one of them */
         x3 = x2 = -a < x1 ? -sqrt(fabs(c / x1)) : sqrt(fabs(c / x1));
         if (pdbl2 != NULL) *pdbl2 = CPL_TRUE;
-        irplib_trace(); /* OK */
     } else if (a < 0.0) {
         /* a and sqrt(Q) have opposite signs - or Q is zero */
         x3 = x2 = sqrtQ - a / 3.0;
         x1 = -c / (x2 * x2);
         if (pdbl2 != NULL) *pdbl2 = CPL_TRUE;
-        irplib_trace(); /* OK */
     } else {
         x1 = -2.0 * sqrtQ;
         x3 = x2 = sqrtQ;
         if (pdbl2 != NULL) *pdbl2 = CPL_TRUE;
-        irplib_trace(); /* OK */
     }
 
     *px1 = x1;
@@ -1000,7 +1002,6 @@ static void irplib_polynomial_solve_1d_3c(double a, double c,
         x3 = 0.5 * CPL_MATH_SQRT3 * fabs(A - B);
 
         x1 = -c / (x2 * x2 + x3 * x3);
-        irplib_trace(); /* OK */
     } else {
         /* A+B and a have opposite signs - or exactly one is zero */
         x1 = A + B - a / 3.0;
@@ -1010,24 +1011,20 @@ static void irplib_polynomial_solve_1d_3c(double a, double c,
         if (x3 > 0.0) {
             /* Real part of complex conjugate */
             x2 = -0.5 * (A + B) - a / 3.0; /* FIXME: Cancellation */
-            irplib_trace(); /* OK */
         } else {
 
             x2 = -a < x1 ? -sqrt(fabs(c / x1)) : sqrt(fabs(c / x1));
             x3 = 0.0;
-            irplib_trace(); /* OK */
         }
     }
 
     if (x3 > 0.0) {
         is_complex = CPL_TRUE;
-        irplib_trace(); /* OK */
     } else {
         /* Whoaa, the imaginary part was truncated to zero
            - return a real, double root */
         x3 = x2;
         if (pdbl2 != NULL) *pdbl2 = CPL_TRUE;
-        irplib_trace(); /* OK */
     }
 
     *px1 = x1;
@@ -1065,9 +1062,9 @@ static void irplib_polynomial_solve_1d_3r(double a, double c,
 
     /* -1.0 <= cos((theta + CPL_MATH_2PI) / 3.0) <= -0.5
        -0.5 <= cos((theta - CPL_MATH_2PI) / 3.0) <=  0.5
-        0.5 <= cos((theta                  ) / 3.0) <=  1.0 */
+        0.5 <= cos((theta               ) / 3.0) <=  1.0 */
 
-#define TR1 (-2.0 * sqrtQ * cos( theta                    / 3.0))
+#define TR1 (-2.0 * sqrtQ * cos( theta                 / 3.0))
 #define TR2 (-2.0 * sqrtQ * cos((theta - CPL_MATH_2PI) / 3.0))
 #define TR3 (-2.0 * sqrtQ * cos((theta + CPL_MATH_2PI) / 3.0))
 
@@ -1088,7 +1085,6 @@ static void irplib_polynomial_solve_1d_3r(double a, double c,
             /* FIXME: Cancellation may still effect x3 ? */
             x3 = TR3 - a / 3.0;
             x2 = -c / ( x1 * x3 );
-            irplib_trace(); /* OK */
         } else {
             /* FIXME: Cancellation may still effect x2, especially
                if x2, x3 is (almost) a double root, i.e.
@@ -1096,24 +1092,20 @@ static void irplib_polynomial_solve_1d_3r(double a, double c,
             x2 = TR2 - a / 3.0;
  
             x3 = -c / ( x1 * x2 );
-            irplib_trace(); /* OK */
         }
     } else if (a < 0.0) {
         x3 = TR3 - a / 3.0;
         if (TR2 < 0.0 && (TR1 + TR2) > 2.0 * a) {
             x1 = TR1 - a / 3.0;
             x2 = -c / ( x1 * x3 );
-            irplib_trace(); /* OK */
         } else {
             x2 = TR2 - a / 3.0;
             x1 = -c / ( x2 * x3 );
-            irplib_trace(); /* OK */
         }
     } else {
         x1 = TR1;
         x2 = TR2;
         x3 = TR3;
-        irplib_trace(); /* OK */
     }
 
     assert(x1 < x3);
@@ -1127,7 +1119,6 @@ static void irplib_polynomial_solve_1d_3r(double a, double c,
            theta is close to PI */
      
         x1 = x2 = 0.5 * ( x1 + x2 );
-        irplib_trace(); /* OK, tested only for x1 == x2 */
     } else if (x2 > x3) {
         /* In absence of round-off:
            theta == 0: x2 == x3,
@@ -1137,7 +1128,6 @@ static void irplib_polynomial_solve_1d_3r(double a, double c,
            Round-off can cause x2 to become greater than x3 */
      
         x3 = x2 = 0.5 * ( x2 + x3 );
-        irplib_trace(); /* OK */
     }
 
     *px1 = x1;
@@ -1215,7 +1205,7 @@ static cpl_error_code irplib_polynomial_solve_1d_4(double p4, double p3,
                 x4 -= 0.25 * p3 / p4;
                 /* Need (only) a guarded swap of x2, x3 */
                 if (x2 > x3) {
-                    IRPLIB_SWAP(x2, x3);
+                    IRPLIB_SWAP_DOUBLE(x2, x3);
                 }
                 if (dbl1) {
                     assert( x1 <= x2); /* The cubic may have 0 as triple root */
@@ -1232,7 +1222,7 @@ static cpl_error_code irplib_polynomial_solve_1d_4(double p4, double p3,
             if (x1 > x2) {
                 assert( x3 <= x2 ); /* Don't swap a complex root */
 
-                IRPLIB_SWAP(x1, x2);
+                IRPLIB_SWAP_DOUBLE(x1, x2);
             } else {
                 assert( x3 >= x2 );
             }
@@ -1390,11 +1380,11 @@ static cpl_error_code irplib_polynomial_solve_1d_4(double p4, double p3,
     return CPL_ERROR_NONE;
 }
 
-#ifdef IPRLIB_POLYNOMIAL_USE_MONOMIAL_ROOT
+#ifdef IRPLIB_POLYNOMIAL_GUESS_ANASOL
 /*----------------------------------------------------------------------------*/
 /**
   @internal
-  @brief    Depress a 1D-polynomial of degree at least 2.
+  @brief    Depress a 1D-polynomial of degree at least 1
   @param    self   The 1D-polynomial to modify
   @return   The mean of the roots of the input polynomial
 
@@ -1407,31 +1397,27 @@ static double irplib_polynomial_depress_1d(cpl_polynomial * self)
     const cpl_size nc1    = degree - 1;
     const double   an     = cpl_polynomial_get_coeff(self, &degree);
     const double   an1    = cpl_polynomial_get_coeff(self, &nc1);
-    double         rmean;
-    cpl_size       i;
+    const double   rmean  = an != 0.0 ? -an1/(an * (double)degree) : 0.0;
 
 
     cpl_ensure(degree > 0,   CPL_ERROR_DATA_NOT_FOUND, 0.0);
 
     assert( an != 0.0 );
 
-    rmean = -an1/(an * (double)degree);
-
     if (rmean != 0.0) {
 
         cpl_polynomial_shift_1d(self, 0, rmean);
 
-        cpl_polynomial_set_coeff(self, &nc1, 0.0); /* Round-off... */
-
     }
 
-    /* Set leading coefficient to one. */
-    for (i = 0; i < degree-1; i++) {
+    /* Divide polynomial by leading coefficient */
+    for (cpl_size i = 0; i < nc1; i++) {
         const double ai = cpl_polynomial_get_coeff(self, &i) / an;
         cpl_polynomial_set_coeff(self, &i, ai);
     }
 
-    cpl_polynomial_set_coeff(self, &degree, 1.0); /* Round-off... */
+    cpl_polynomial_set_coeff(self, &nc1,    0.0); /* Ensure exact values */
+    cpl_polynomial_set_coeff(self, &degree, 1.0); /* Ensure exact values */
 
     return rmean;
 }
@@ -1464,8 +1450,6 @@ cpl_error_code irplib_polynomial_divide_1d_root(cpl_polynomial * p, double r,
 
 
     cpl_ensure_code(p != NULL, CPL_ERROR_NULL_INPUT);
-    cpl_ensure_code(cpl_polynomial_get_dimension(p) == 1,
-                    CPL_ERROR_INVALID_TYPE);
     cpl_ensure_code(n > 0, CPL_ERROR_DATA_NOT_FOUND);
 
     sum = cpl_polynomial_get_coeff(p, &n);
