@@ -1,7 +1,6 @@
-/* $Id: irplib_spectrum.c,v 1.29 2012-01-12 11:50:41 llundin Exp $
- *
+/*
  * This file is part of the irplib package
- * Copyright (C) 2002,2003 European Southern Observatory
+ * Copyright (C) 2002,2003,2014 European Southern Observatory
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,13 +17,6 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02111-1307  USA
  */
 
-/*
- * $Author: llundin $
- * $Date: 2012-01-12 11:50:41 $
- * $Revision: 1.29 $
- * $Name: not supported by cvs2svn $
- */
-
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
@@ -33,24 +25,17 @@
                                 Includes
  -----------------------------------------------------------------------------*/
 
+#include "irplib_wlxcorr.h"
+#include "irplib_spectrum.h"
+
 #include <math.h>
 #include <float.h>
 #include <cpl.h>
 
-#include "irplib_wlxcorr.h"
-#include "irplib_spectrum.h"
-
 /*-----------------------------------------------------------------------------
                                    Define
  -----------------------------------------------------------------------------*/
 
-/* TEMPORARY SUPPORT OF CPL 5.x */
-#ifndef CPL_SIZE_FORMAT
-#define CPL_SIZE_FORMAT "d"
-#define cpl_size int
-#endif
-/* END TEMPORARY SUPPORT OF CPL 5.x */
-
 #define SPECTRUM_HW                     16
 #define MIN_THRESH_FACT                 0.9
 #define MAX_THRESH_FACT                 1.1
@@ -283,9 +268,6 @@ cpl_vector * irplib_spectrum_detect_peaks(
     cpl_vector      *   spec_convolved ;
     double          *   pspec_convolved ;
     int                 filt_size ;
-    cpl_vector      *   conv_kernel ;
-    cpl_vector      *   extract ;
-    cpl_vector      *   extract_x ;
     cpl_vector      *   big_detected ;
     cpl_vector      *   big_fwhms ;
     cpl_vector      *   big_area ;
@@ -293,12 +275,9 @@ cpl_vector * irplib_spectrum_detect_peaks(
     double          *   pbig_fwhms ;
     double          *   pbig_area ;
     cpl_vector      *   detected ;
-    double          *   pdetected ;
     cpl_vector      *   fwhms ;
-    double          *   pfwhms ;
     cpl_vector      *   area ;
-    double          *   parea ;
-    double              max, med, stdev, cur_val ;
+    double              max, med, stdev ;
     double              x0, sig, norm, offset ;
     int                 nb_det, nb_samples, hwidth, start, stop ;
     int                 i, j ;
@@ -312,7 +291,7 @@ cpl_vector * irplib_spectrum_detect_peaks(
     hwidth = 5 ;
 
     /* Subtract the low frequency part */
-    cpl_msg_info(__func__, "Low Frequency signal removal") ;
+    cpl_msg_debug(__func__, "Low Frequency signal removal") ;
     if ((filtered=cpl_vector_filter_median_create(in, filt_size))==NULL){
         cpl_msg_error(__func__, "Cannot filter the spectrum") ;
         return NULL ;
@@ -331,7 +310,8 @@ cpl_vector * irplib_spectrum_detect_peaks(
     /* Convolve */
     spec_convolved = cpl_vector_duplicate(spec_clean) ;
     if (fwhm > 0) {
-        cpl_msg_info(__func__, "Spectrum convolution") ;
+        cpl_vector * conv_kernel ;
+        cpl_msg_debug(__func__, "Spectrum convolution") ;
         /* Create convolution kernel */
         if ((conv_kernel = irplib_wlxcorr_convolve_create_kernel(fwhm,
                         fwhm)) == NULL) {
@@ -380,6 +360,10 @@ cpl_vector * irplib_spectrum_detect_peaks(
     /* Loop on the detected lines */
     nb_det = 0 ;
     while (max > med + stdev * sigma) {
+        cpl_vector * extract ;
+        cpl_vector * extract_x ;
+        double       cur_val ;
+
         /* Compute the position */
         i=0 ;
         while (pspec_convolved[i] < max) i++ ;
@@ -399,7 +383,7 @@ cpl_vector * irplib_spectrum_detect_peaks(
         if (cpl_vector_fit_gaussian(extract_x, NULL, extract, NULL, 
                     CPL_FIT_ALL, &x0, &sig, &norm, &offset, NULL, NULL, 
                     NULL) != CPL_ERROR_NONE) {
-            cpl_msg_warning(__func__, 
+            cpl_msg_debug(__func__, 
                     "Cannot fit a gaussian at [%d, %d]",
                     start, stop) ;
             cpl_error_reset() ;
@@ -447,6 +431,9 @@ cpl_vector * irplib_spectrum_detect_peaks(
         area = NULL ;
         fwhms = NULL ;
     } else {
+        double * pdetected ;
+        double * pfwhms ;
+        double * parea ;
         detected = cpl_vector_new(nb_det) ;
         area = cpl_vector_new(nb_det) ;
         fwhms = cpl_vector_new(nb_det) ;
