@@ -118,8 +118,7 @@ static int hawki_cal_distortion_fit_first_order_solution
  cpl_polynomial   * fit2d_y);
 
 static cpl_propertylist ** hawki_cal_distortion_qc
-(hawki_distortion ** distortion,
- int              *  nmatched_pairs,
+(int              *  nmatched_pairs,
  double           *  rms);
 
 static int hawki_cal_distortion_save
@@ -456,7 +455,7 @@ static int hawki_cal_distortion(cpl_parameterlist *   parlist,
     }
     
     /* Get some QC */
-    qclists =  hawki_cal_distortion_qc(distortion, nmatched_pairs, rms);
+    qclists =  hawki_cal_distortion_qc(nmatched_pairs, rms);
     
     /* Save the products */
     cpl_msg_info(__func__,"Saving products");
@@ -1273,8 +1272,7 @@ static int hawki_cal_distortion_fit_first_order_solution
  */ 
 /*----------------------------------------------------------------------------*/
 static cpl_propertylist ** hawki_cal_distortion_qc
-(hawki_distortion ** distortion,
- int              *  nmatched_pairs,
+(int              *  nmatched_pairs,
  double           *  rms)
 {
     int idet;
