@@ -34,6 +34,7 @@
  -----------------------------------------------------------------------------*/
 
 #include <string.h>
+#include <locale.h>
 #include <math.h>
 #include <cpl.h>
 
@@ -45,23 +46,18 @@
 #include "irplib_wcs.h"
 
 #include "hawki_image_stats.h"
-#include "hawki_utils.h"
+#include "hawki_utils_legacy.h"
 #include "hawki_calib.h"
 #include "hawki_load.h"
 #include "hawki_save.h"
-#include "hawki_pfits.h"
-#include "hawki_dfs.h"
+#include "hawki_pfits_legacy.h"
+#include "hawki_dfs_legacy.h"
 #include "hawki_alloc.h"
 
 /*-----------------------------------------------------------------------------
                             Functions prototypes
  -----------------------------------------------------------------------------*/
 
-#ifdef __cplusplus
-extern "C"
-#endif
-int cpl_plugin_get_info(cpl_pluginlist * list);
-
 static int hawki_cal_zpoint_create(cpl_plugin *) ;
 static int hawki_cal_zpoint_exec(cpl_plugin *) ;
 static int hawki_cal_zpoint_destroy(cpl_plugin *) ;
@@ -168,7 +164,7 @@ static struct {
 } hawki_cal_zpoint_outputs;
 
 static char hawki_cal_zpoint_description[] =
-"hawki_cal_zpoint -- Zero point recipe\n"
+"(OBSOLETE) hawki_cal_zpoint -- Zero point recipe\n"
 "The input of the recipe files listed in the Set Of Frames (sof-file)\n"
 "must be tagged as:\n"
 "raw-file.fits "HAWKI_CAL_ZPOINT_RAW" or\n"
@@ -206,11 +202,11 @@ int cpl_plugin_get_info(cpl_pluginlist * list)
                     HAWKI_BINARY_VERSION,
                     CPL_PLUGIN_TYPE_RECIPE,
                     "hawki_cal_zpoint",
-                    "Zero point computation recipe",
+                    "(OBSOLETE) Zero point computation recipe",
                     hawki_cal_zpoint_description,
-                    "Cesar Enrique Garcia Dabo",
+                    "ESO Pipeline Group",
                     PACKAGE_BUGREPORT,
-                    hawki_get_license(),
+                    hawki_get_license_legacy(),
                     hawki_cal_zpoint_create,
                     hawki_cal_zpoint_exec,
                     hawki_cal_zpoint_destroy) ;
@@ -398,7 +394,7 @@ static int hawki_cal_zpoint(
     }
 
     /* Identify the RAW and CALIB frames in the input frameset */
-    if (hawki_dfs_set_groups(framelist)) {
+    if (hawki_dfs_set_groups_legacy(framelist)) {
         cpl_msg_error(__func__, "Cannot identify RAW and CALIB frames") ;
         return -1 ;
     }
@@ -562,7 +558,7 @@ static cpl_table ** hawki_cal_zpoint_reduce
     error_prevstate = cpl_errorstate_get();
     cur_frame = cpl_frameset_get_position(set, 0) ;
     plist=cpl_propertylist_load(cpl_frame_get_filename(cur_frame), 0) ;
-    if ((sval = hawki_pfits_get_filter(plist)) == NULL) return NULL ;
+    if ((sval = hawki_pfits_get_filter_legacy(plist)) == NULL) return NULL ;
     else sprintf(hawki_cal_zpoint_outputs.filter, "%s", sval) ;
     if (hawki_cal_zpoint_config.target_ra > 998.0)
     {
@@ -586,7 +582,7 @@ static cpl_table ** hawki_cal_zpoint_reduce
             return NULL;
         }
     }
-    hawki_cal_zpoint_outputs.dit = hawki_pfits_get_dit(plist) ;
+    hawki_cal_zpoint_outputs.dit = hawki_pfits_get_dit_legacy(plist) ;
     hawki_cal_zpoint_outputs.pixscale = hawki_pfits_get_pixscale(plist) ;
     cpl_propertylist_delete(plist) ;
     if(!cpl_errorstate_is_equal(error_prevstate))
@@ -1504,7 +1500,7 @@ static int hawki_cal_zpoint_save
     /* Free */
     if(!cpl_errorstate_is_equal(error_prevstate))
     {
-        cpl_errorstate_set(CPL_ERROR_NONE);
+        cpl_error_reset();
         return -1;
     }
     return  0;
@@ -1866,6 +1862,7 @@ int hawki_cal_zpoint_retrieve_input_param
     /* --xcoord */
     par = cpl_parameterlist_find(parlist, "hawki.hawki_cal_zpoint.xcoord");
     sval = cpl_parameter_get_string(par);
+    setlocale(LC_NUMERIC, "C"); // Behaviour of sscanf depends on the locale
     if (sscanf(sval, "%lf,%lf,%lf,%lf",
                hawki_cal_zpoint_config.xcoord,
                hawki_cal_zpoint_config.xcoord+1,
