@@ -473,13 +473,6 @@ static int hawki_dark_combine(cpl_parameterlist *parlist,
         return(-1);
     }
 
-    /* Check the files in the frameset */
-
-    if (casu_frameset_fexists(framelist) != CASU_OK) {
-        cpl_msg_error(fctid,"Input frameset is missing files. Check SOF");
-        return(-1);
-    }
-
     /* Initialise some things */
 
     cs.we_expect = 0;
