@@ -1203,6 +1203,8 @@ static int hawki_sci_jitter_sky_running
 
     /* Test entries */
     if (in==NULL) return -1;
+    if(bkg_stats == NULL) return -1;
+    if(bkg_stats[idet] == NULL) return -1;
 
     /* Initialise */
     rejmin = hawki_sci_jitter_config.sky_rejmin ;
@@ -1245,15 +1247,12 @@ static int hawki_sci_jitter_sky_running
 
         /* Subtract the background from the current image */
         tmp_ima = cpl_image_subtract_create(cpl_imagelist_get(in, k), bkg);
-        
+
         /* Statistics on the background */
-        if(bkg_stats != NULL)
-        {
-            hawki_image_stats_fill_from_image
-                (bkg_stats, bkg,
-                 1, 1, nx, ny,
-                 idet, k);
-        }
+        hawki_image_stats_fill_from_image
+        (bkg_stats, bkg,
+         1, 1, nx, ny,
+         idet, k);
         cpl_image_delete(bkg);
        
         /* Place the new image in a result buffer */
