@@ -38,9 +38,9 @@
 
 #include "irplib_utils.h"
 
-#include "hawki_utils.h"
-#include "hawki_pfits.h"
-#include "hawki_dfs.h"
+#include "hawki_utils_legacy.h"
+#include "hawki_pfits_legacy.h"
+#include "hawki_dfs_legacy.h"
 #include "hawki_load.h"
 #include "hawki_save.h"
 #include "hawki_distortion.h"
@@ -49,11 +49,6 @@
                             Functions prototypes
  -----------------------------------------------------------------------------*/
 
-#ifdef __cplusplus
-extern "C"
-#endif
-int cpl_plugin_get_info(cpl_pluginlist * list);
-
 static int hawki_step_apply_dist_create(cpl_plugin *) ;
 static int hawki_step_apply_dist_exec(cpl_plugin *) ;
 static int hawki_step_apply_dist_destroy(cpl_plugin *) ;
@@ -77,7 +72,7 @@ int hawki_step_apply_dist_compute_and_save
  -----------------------------------------------------------------------------*/
 
 static char hawki_step_apply_dist_description[] =
-"hawki_step_apply_dist -- Distortion correction utility\n"
+"(OBSOLETE) hawki_step_apply_dist -- Distortion correction utility\n"
 "This recipe accepts three types of frames:\n"
 "object.fits  Images to correct (PRO.CATG = "HAWKI_CALPRO_BKG_SUBTRACTED")\n"
 "distmap_x.fits The image with the distortion in X.\n"
@@ -113,11 +108,11 @@ int cpl_plugin_get_info(cpl_pluginlist * list)
                     HAWKI_BINARY_VERSION,
                     CPL_PLUGIN_TYPE_RECIPE,
                     "hawki_step_apply_dist",
-                    "Distortion correction utility",
+                    "(OBSOLETE) Distortion correction utility",
                     hawki_step_apply_dist_description,
-                    "Cesar Enrique Garcia Dabo",
+                    "ESO Pipeline Group",
                     PACKAGE_BUGREPORT,  
-                    hawki_get_license(),
+                    hawki_get_license_legacy(),
                     hawki_step_apply_dist_create,
                     hawki_step_apply_dist_exec,
                     hawki_step_apply_dist_destroy) ;
@@ -213,7 +208,7 @@ static int hawki_step_apply_dist(
     /* Retrieve input parameters */
  
     /* Identify the RAW and CALIB frames in the input frameset */
-    if (hawki_dfs_set_groups(frameset)) {
+    if (hawki_dfs_set_groups_legacy(frameset)) {
         cpl_msg_error(__func__, "Cannot identify RAW and CALIB frames") ;
         return -1 ;
     }
@@ -411,7 +406,7 @@ int hawki_step_apply_dist_compute_and_save
                 (object_images, iframe, 
                  parlist, used_frameset, recipe_frameset) == -1)
         {
-            cpl_errorstate_set(CPL_ERROR_NONE);
+            cpl_error_reset();
         }
         
         /* Free resources */
