@@ -26,18 +26,19 @@
  -----------------------------------------------------------------------------*/
 
 #include <math.h>
+#include <locale.h>
 #include <cpl.h>
 #include <string.h>
 
 #include "irplib_utils.h"
 #include "irplib_calib.h"
 
-#include "hawki_utils.h"
+#include "hawki_utils_legacy.h"
 #include "hawki_calib.h"
 #include "hawki_load.h"
 #include "hawki_save.h"
-#include "hawki_pfits.h"
-#include "hawki_dfs.h"
+#include "hawki_pfits_legacy.h"
+#include "hawki_dfs_legacy.h"
 #include "hawki_saa.h"
 #include "hawki_bkg.h"
 #include "hawki_distortion.h"
@@ -48,11 +49,6 @@
                             Functions prototypes
  -----------------------------------------------------------------------------*/
 
-#ifdef __cplusplus
-extern "C"
-#endif
-int cpl_plugin_get_info(cpl_pluginlist * list);
-
 static int hawki_step_combine_create(cpl_plugin *) ;
 static int hawki_step_combine_exec(cpl_plugin *) ;
 static int hawki_step_combine_destroy(cpl_plugin *) ;
@@ -105,7 +101,7 @@ static struct
 } hawki_step_combine_output;
 
 static char hawki_step_combine_description[] =
-"hawki_step_combine -- hawki combine jitter images.\n"
+"(OBSOLETE) hawki_step_combine -- hawki combine jitter images.\n"
 "The files listed in the Set Of Frames (sof-file) must be tagged:\n"
 "science-file.fits "HAWKI_CALPRO_DIST_CORRECTED" or\n"
 "science-file.fits "HAWKI_CALPRO_BKG_SUBTRACTED" or\n"
@@ -150,11 +146,11 @@ int cpl_plugin_get_info(cpl_pluginlist * list)
                     HAWKI_BINARY_VERSION,
                     CPL_PLUGIN_TYPE_RECIPE,
                     "hawki_step_combine",
-                    "Jitter image combination recipe",
+                    "(OBSOLETE) Jitter image combination recipe",
                     hawki_step_combine_description,
-                    "Cesar Enrique Garcia Dabo",
+                    "ESO Pipeline Group",
                     PACKAGE_BUGREPORT,  
-                    hawki_get_license(),
+                    hawki_get_license_legacy(),
                     hawki_step_combine_create,
                     hawki_step_combine_exec,
                     hawki_step_combine_destroy) ;
@@ -316,7 +312,7 @@ static int hawki_step_combine(
     }
 
     /* Identify the RAW and CALIB frames in the input frameset */
-    if (hawki_dfs_set_groups(framelist)) {
+    if (hawki_dfs_set_groups_legacy(framelist)) {
         cpl_msg_error(__func__, "Cannot identify RAW and CALIB frames") ;
         return -1 ;
     }
@@ -475,6 +471,7 @@ int hawki_step_combine_retrieve_input_param
     par = cpl_parameterlist_find(parlist, 
             "hawki.hawki_step_combine.rej");
     sval = cpl_parameter_get_string(par);
+    setlocale(LC_NUMERIC, "C"); // Behaviour of sscanf depends on the locale
     if (sscanf(sval, "%d,%d",
                &hawki_step_combine_config.rej_low,
                &hawki_step_combine_config.rej_high)!=2)
@@ -1014,7 +1011,7 @@ static int hawki_step_combine_save
 
     if(!cpl_errorstate_is_equal(error_prevstate))
     {
-        cpl_errorstate_set(CPL_ERROR_NONE);
+        cpl_error_reset();
         return 1;
     }
 
