@@ -34,18 +34,19 @@
  -----------------------------------------------------------------------------*/
 
 #include <math.h>
+#include <locale.h>
 #include <cpl.h>
 #include <string.h>
 
 #include "irplib_utils.h"
 #include "irplib_calib.h"
 
-#include "hawki_utils.h"
+#include "hawki_utils_legacy.h"
 #include "hawki_calib.h"
 #include "hawki_load.h"
 #include "hawki_save.h"
-#include "hawki_pfits.h"
-#include "hawki_dfs.h"
+#include "hawki_pfits_legacy.h"
+#include "hawki_dfs_legacy.h"
 #include "hawki_saa.h"
 #include "hawki_bkg.h"
 #include "hawki_distortion.h"
@@ -63,11 +64,6 @@
                             Functions prototypes
  -----------------------------------------------------------------------------*/
 
-#ifdef __cplusplus
-extern "C"
-#endif
-int cpl_plugin_get_info(cpl_pluginlist * list);
-
 static int hawki_step_refine_offsets_create(cpl_plugin *) ;
 static int hawki_step_refine_offsets_exec(cpl_plugin *) ;
 static int hawki_step_refine_offsets_destroy(cpl_plugin *) ;
@@ -109,7 +105,7 @@ static struct
 } hawki_step_refine_offsets_config;
 
 static char hawki_step_refine_offsets_description[] =
-"hawki_step_refine_offsets -- utility to refine the nominal offsets.\n"
+"(OBSOLETE) hawki_step_refine_offsets -- utility to refine the nominal offsets.\n"
 "This utility will take the offsets in the header as a first approach\n"
 "and tries to refine them correlating the input images at the points\n"
 "given by the detected objects.\n"
@@ -148,11 +144,11 @@ int cpl_plugin_get_info(cpl_pluginlist * list)
                     HAWKI_BINARY_VERSION,
                     CPL_PLUGIN_TYPE_RECIPE,
                     "hawki_step_refine_offsets",
-                    "Jitter recipe",
+                    "(OBSOLETE) Jitter recipe",
                     hawki_step_refine_offsets_description,
-                    "Cesar Enrique Garcia Dabo",
+                    "ESO Pipeline Group",
                     PACKAGE_BUGREPORT,  
-                    hawki_get_license(),
+                    hawki_get_license_legacy(),
                     hawki_step_refine_offsets_create,
                     hawki_step_refine_offsets_exec,
                     hawki_step_refine_offsets_destroy) ;
@@ -280,7 +276,7 @@ static int hawki_step_refine_offsets(
     }
 
     /* Identify the RAW and CALIB frames in the input frameset */
-    if (hawki_dfs_set_groups(framelist)) {
+    if (hawki_dfs_set_groups_legacy(framelist)) {
         cpl_msg_error(__func__, "Cannot identify RAW and CALIB frames") ;
         return -1 ;
     }
@@ -394,6 +390,7 @@ int hawki_step_refine_offsets_retrieve_input_param
     par = cpl_parameterlist_find
         (parlist, "hawki.hawki_step_refine_offsets.xcorr");
     sval = cpl_parameter_get_string(par);
+    setlocale(LC_NUMERIC, "C"); // Behaviour of sscanf depends on the locale
     if (sscanf(sval, "%d,%d,%d,%d",
                &hawki_step_refine_offsets_config.sx,
                &hawki_step_refine_offsets_config.sy,
@@ -661,7 +658,7 @@ static int hawki_step_refine_offsets_save
     cpl_free(offset_tables);
     if(!cpl_errorstate_is_equal(error_prevstate))
     {
-        cpl_errorstate_set(CPL_ERROR_NONE);
+        cpl_error_reset();
         return -1;
     }
     return  0;
