@@ -532,13 +532,6 @@ static int hawki_twilight_flat_combine(cpl_parameterlist *parlist,
         return(-1);
     }
 
-    /* Check the files in the frameset */
-
-    if (casu_frameset_fexists(framelist) != CASU_OK) {
-        cpl_msg_error(fctid,"Input frameset is missing files. Check SOF");
-        return(-1);
-    }
-
     /* Initialise some things */
 
     hawki_twilight_flat_combine_init(&ps);
