@@ -1,4 +1,6 @@
 from __future__ import with_statement
+from __future__ import absolute_import
+from __future__ import print_function
 import sys
 
 
@@ -16,7 +18,7 @@ try:
     
 except ImportError:
     import_success = False
-    print "Error importing modules pyfits, wx, matplotlib, numpy"
+    print("Error importing modules pyfits, wx, matplotlib, numpy")
 
 # Median absolute deviation function; used to scale the images
 def MAD(x):
@@ -249,8 +251,8 @@ class DataPlotterManager(object):
             self.right_opts = {"Click to\nadvance to\nnext item\n(if available)"}
 
             # Set the initial radio button selections (value 1 for left, 0 for mid)
-            self.left_label = [key for key, value in self.left_opts.iteritems() if value == 1][0]
-            self.mid_label = [key for key, value in self.mid_opts.iteritems() if value == 0][0]
+            self.left_label = [key for key, value in self.left_opts.items() if value == 1][0]
+            self.mid_label = [key for key, value in self.mid_opts.items() if value == 0][0]
 
         else:
             # Set the plotting functions to NODATA ones
